"""
This script generates a project documentation.
It takes at least three arguments :
- an output format : the format of generated files ;
- an output directory : the directory where the files will be generate ;
- some XML files : files which describe the project.
"""

import xml.parsers.expat
import os.path
import sys


TXT_OUTPUT = 'TXT'

HTML_OUTPUT = 'HTML'

TXT_WIDTH = 75

HTML_HEADER = '''
<HTML>
<HEAD>
<TITLE>Maestro project Web site</TITLE>
</HEAD>
<BODY BGCOLOR="white">
<IMG SRC="images/maestro-logo.png" STYLE="width:292px;height:78px" />
<DIV STYLE="background-color:lightblue">
<A HREF="index.html">Home</A>&nbsp;-&nbsp;
<A HREF="features.html">Features</A>&nbsp;-&nbsp;
<A HREF="shots.html">Screenshots</A>&nbsp;-&nbsp;
<A HREF="download.html">Download</A>&nbsp;-&nbsp;
<A HREF="faq.html">FAQ</A>
</DIV>
'''

HTML_FOOTER = '''
</BODY>
</HTML>
'''

class OutputDocument:
    """A base output document."""

    __file = None

    def __init__(self, p_file):
        self.__file = p_file

    def write(self, p_text):
        self.__file.write(p_text)

    def write_text(self, p_text):
        self.write(p_text)

    def write_start_section(self):
        pass

    def write_end_section(self):
        pass

    def write_start_title(self):
        pass

    def write_end_title(self):
        pass

    def write_start_paragraph(self):
        pass

    def write_end_paragraph(self):
        pass

    def write_start_list(self):
        pass

    def write_end_list(self):
        pass

    def write_start_item(self):
        pass

    def write_end_item(self):
        pass

    def write_start_people(self, p_mail):
        pass

    def write_end_people(self):
        pass

    def write_link(self, p_text, p_url):
        pass

    def write_start_code(self):
        pass

    def write_end_code(self):
        pass

    def write_image(self, p_src, p_title, p_width, p_height):
        pass

    def close(self):
        self.__file.close()


class TextDocument(OutputDocument):
    """A text document."""

    __len = 0

    __mail = ''

    def __init__(self, p_file):
        OutputDocument.__init__(self, p_file)
        self.__len = 0
        self.__mail = ''

    def write(self, p_text):
#        print 'Text to write :%s\\' % p_text
        _words = p_text.split(' ')
        _len = len(_words)
        for _cpt in range(_len):
            _word = _words[_cpt]
            if self.__len + len(_word) > TXT_WIDTH:
#                OutputDocument.write(self, '+' + str(self.__len) + '+' + str(len(_word)))
                self.__write_nl()
            # write the word
            OutputDocument.write(self, _word)
            self.__len += len(_word)
            # if it is not the last word, write a white space
            if _cpt < _len - 1:
                OutputDocument.write(self, ' ')
                self.__len += 1
#        OutputDocument.write(self, '*')

    def __write_nl(self):
        OutputDocument.write(self, '\n')
        self.__len = 0

    """
    def write_text(self, p_text):
        self.write('|')
        self.__len += len(p_text)
        if self.__len > 70:
            self.__write_nl()
            self.write('+')
            self.__len = len(p_text)
        OutputDocument.write_text(self, p_text)
    """

    def write_end_section(self):
        self.__write_nl()

    def write_start_title(self):
        self.__write_nl()

    def write_end_title(self):
        self.__write_nl()
        self.write('-' * TXT_WIDTH)
        self.__write_nl()
#        self.__write_nl()

    def write_start_paragraph(self):
        self.__write_nl()

    def write_end_paragraph(self):
        self.__write_nl()

    def write_start_list(self):
        self.__write_nl()

    def write_end_list(self):
        self.__write_nl()

    def write_start_item(self):
        self.__write_nl()
        OutputDocument.write(self, ' - ')

#    def write_end_item(self):
#        self.__write_nl()

    def write_start_people(self, p_mail):
        self.__mail = p_mail

    def write_end_people(self):
        self.write(' <' + self.__mail + '>')

    def write_start_code(self):
        self.__write_nl()
        self.write(' ' * 5)

    def write_link(self, p_text, p_url):
        self.write(p_url)

    def write_image(self, p_src, p_title, p_width, p_height):
        self.write(p_title)



class HTMLDocument(OutputDocument):
    """A HTML document."""

    def __init__(self, p_file):
        OutputDocument.__init__(self, p_file)
        # write header
        self.write(HTML_HEADER)
        """
        try:
            _header_file = open('header.html', 'r')
        except:
            pass
        else:
            _lines = _header_file.readlines()
            for _line in _lines:
                self.write(_line)
        """

#    def write_text(self, p_text):
#        OutputDocument.write(self, p_text)

    def write_start_title(self):
        self.write('\n<H2>')

    def write_end_title(self):
        self.write('</H2>')

    def write_start_paragraph(self):
        self.write('\n<P>')

    def write_end_paragraph(self):
        self.write('</P>')

    def write_start_list(self):
        self.write('<UL>')

    def write_end_list(self):
        self.write('\n</UL>\n')

    def write_start_item(self):
        self.write('\n<LI>')

    def write_end_item(self):
        self.write('</LI>')

    def write_start_people(self, p_mail):
        self.write('\n<A HREF="mailto:' + p_mail + '">')

    def write_end_people(self):
        self.write('</A>')

    def write_link(self, p_text, p_url):
        self.write('\n<A HREF="' + p_url + '">' + p_text + '</A>')

    def write_start_code(self):
        self.write('\n<PRE>')

    def write_end_code(self):
        self.write('</PRE>')

    def write_image(self, p_src, p_title, p_width, p_height):
        self.write('\n<IMG SRC="' + p_src + '" TITLE="' + p_title + '" WIDTH=' + p_width + ' HEIGHT=' + p_height + ' />')
        self.write('\n<DIV STYLE="text-align:center; width:' + p_width + '">')
        self.write('\n<EM>' + p_title + '</EM>')
        self.write('\n</DIV>')
        self.write('\n<BR/>')
                   

    def close(self):
        # write footer
        self.write(HTML_FOOTER)
        """
        try:
            _footer_file = open('footer.html', 'r')
        except:
            pass
        else:
            _lines = _footer_file.readlines()
            for _line in _lines:
                self.write(_line)
        """
        # close document
        OutputDocument.close(self)


def open_text_document(p_file_path):
    _file = open(p_file_path, 'w')
    _writer = TextDocument(_file)
    return _writer

def open_html_document(p_file_path):
    _file = open(p_file_path, 'w')
    _writer = HTMLDocument(_file)
    return _writer


class DocumentParser:
    """A XML document parser."""

    __format = ''

    __dir_path = '.'

    __parser = None

    __documents = {}

    __current_doc = None

    __len = 0

    def __init__(self, p_output_format, p_dir_path):
        self.__format = p_output_format
        self.__dir_path = p_dir_path
        self.__parser = xml.parsers.expat.ParserCreate('ISO-8859-1')
#        self.__parser.returns_unicode = 0
        self.__parser.StartElementHandler = self.__start_element
        self.__parser.EndElementHandler = self.__end_element
        self.__parser.CharacterDataHandler = self.__char_data
        self.__documents = {}
        self.__current_doc = OutputDocument(sys.stderr)
        self.__len = 0


    def parse(self, p_file_path):
        """Parse the given file."""
        _f = open(p_file_path, 'r')
        # parse the file
        self.__parser.ParseFile(_f)
        # get the error code
        _num = self.__parser.ErrorCode
        if _num != 0:
            _msg = xml.parsers.expat.ErrorString(_num)
            _line = self.__parser.ErrorLineNumber
            _col = self.__parser.ErrorColumnNumber
            print 'Parse error : %s %d %d : %d --> %s' % (p_file_path, _line, _col, _num, _msg)
        # close file
        _f.close()

    def __get_document(self, p_output_id):
        if not self.__documents.has_key(p_output_id):
            _path = os.path.join(self.__dir_path, p_output_id)
            if self.__format == HTML_OUTPUT:
                _doc = open_html_document(_path)
            else:
                _doc = open_text_document(_path)
            self.__documents[p_output_id] = _doc
        _doc = self.__documents[p_output_id]
        return _doc

    def __start_element(self, p_name, p_attrs):
        """Handle a start element."""
#        print 'Start element:', p_name, p_attrs
        if p_name == 'section':
            if self.__format == HTML_OUTPUT:
                if p_attrs.has_key('html'):
                    self.__current_doc = self.__get_document(p_attrs['html'])
                else:
                    self.__current_doc = OutputDocument(sys.stderr)
            else:
                if p_attrs.has_key('txt'):
                    self.__current_doc = self.__get_document(p_attrs['txt'])
                else:
                    self.__current_doc = OutputDocument(sys.stderr)
            self.__current_doc.write_start_section()
        elif p_name == 'title':
            self.__current_doc.write_start_title()
        elif p_name == 'para':
            self.__current_doc.write_start_paragraph()
        elif p_name == 'list':
            self.__current_doc.write_start_list()
        elif p_name == 'item':
            self.__current_doc.write_start_item()
        elif p_name == 'people':
            self.__current_doc.write_start_people(p_attrs['mail'])
        elif p_name == 'code':
            self.__current_doc.write_start_code()
        elif p_name == 'link':
            _url = p_attrs['url']
            _text = _url
            if p_attrs.has_key('type') and p_attrs['type'] == 'file':
                _text = os.path.basename(_url)
            self.__current_doc.write_link(_text, _url)
        elif p_name == 'image':
            self.__current_doc.write_image(p_attrs['src'], p_attrs['title'], p_attrs['width'], p_attrs['height'])
            
    
    def __end_element(self, p_name):
        """Handle an end element."""
#        print 'End element:', p_name
        if p_name == 'section':
            self.__current_doc.write_end_section()
            self.__current_doc = OutputDocument(sys.stderr)
        elif p_name == 'title':
            self.__current_doc.write_end_title()
        elif p_name == 'para':
            self.__current_doc.write_end_paragraph()
        elif p_name == 'list':
            self.__current_doc.write_end_list()
        elif p_name == 'item':
            self.__current_doc.write_end_item()
        elif p_name == 'people':
            self.__current_doc.write_end_people()
        elif p_name == 'code':
            self.__current_doc.write_end_code()
#        elif p_name == 'link':
#            self.__current_doc.write_end_link()

    def __char_data(self, p_data):
        """Handle a character data."""
#        print 'Character data:', repr(p_data)
        _data = p_data.encode("ISO-8859-1")
        _data = _data.replace('\n', '')
        self.__current_doc.write_text(_data)


    def write_documents(self):
        _keys = self.__documents.keys()
        for _key in _keys:
            self.__documents[_key].close()
        return len(self.__documents)


if __name__ == '__main__':
    if len(sys.argv) < 4:
        print 'Usage : %s %s output_format output_dir xml_file' % (sys.executable, sys.argv[0])
    else:
        _format = sys.argv[1]
        if _format != TXT_OUTPUT and _format != HTML_OUTPUT:
            print 'Unrecognize output format : %s' % _format
            print 'Availlable output formats : %s, %s' % (TXT_OUTPUT, HTML_OUTPUT)
        else:
            _dir_path = sys.argv[2]
            _parser = DocumentParser(_format, _dir_path)
            for _path in sys.argv[3:]:
                _parser.parse(_path)
            _nb = _parser.write_documents()
            print '%d document(s) written' % _nb
