# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from util.String import String

ARTIST_TAG = 'artist'

ARTIST_NAME_TAG = 'name'

FEAT_ARTIST_NAME_TAG = 'feat_name'

NAMES_SEP = ', '

FEAT_SEP = ' feat. '

class Artist:
    """A band, a set of artists and/or featuring artists.

    Warning ! For a reason of performance, this class is not synchronized.
    Don't apply its modificators to a shared instance.
    """

    __artist_names = []

    __feat_artist_names = []

#    __name = String("")

    def __init__(self, p_name):
        "Construct an artist."
#        self.__name = String(p_name)
        self.__artist_names = []
        self.__feat_artist_names = []
        self.add_name(p_name)

#    def get_name(self):
#        """Return the name."""
#        return self.__name.data

#    def set_name(self, p_name):
#        """Set the given name."""
#        self.__name = String(p_name)

    def add_name(self, p_name):
        """Add the given artist name to the artist."""
#        self.__name = String(p_name)
        _name = String(p_name)
        self.__artist_names.append(_name)

    def add_feat_name(self, p_name):
        """Add the given featuring artist name to the artist."""
        _name = String(p_name)
        self.__feat_artist_names.append(_name)

    def get_names_number(self):
        """Return the number of artist names."""
        return len(self.__artist_names)

    def get_name(self, p_index):
        """Return the artist name at the given index."""
        return str(self.__artist_names[p_index])

    def get_feat_names_number(self):
        """Return the number of featuring artist names."""
        return len(self.__feat_artist_names)

    def get_feat_name(self, p_index):
        """Return the featuring artist name at the given index."""
        return str(self.__feat_artist_names[p_index])

    def get_main_artist(self):
        """Return the main artist."""
        _artist = Artist(self.__artist_names[0].data)
        return _artist

    def clone(self):
        """Return a copy of the artist."""
#        _artist = Artist(self.__name.data)
        # create a new artist
        _artist = Artist(self.__artist_names[0].data)
        # add all artist names
        for _name in self.__artist_names[1:]:
            _artist.add_name(_name.data)
        # add all featuring artist names
        for _name in self.__feat_artist_names:
            _artist.add_feat_name(_name.data)
        return _artist

    def matchs(self, p_artist):
        """Match with another artist."""
        _ret = 1
        if p_artist is not None:
#            _ret = self.__name.matchs(p_artist.get_name())
            for _name1 in p_artist.__artist_names:
                if _ret:
                    _ret = 0
                    for _name2 in self.__artist_names:
                        if _name2.matchs(str(_name1)):
                            _ret = 1
                            break
            for _name1 in p_artist.__feat_artist_names:
                if _ret:
                    _ret = 0
                    for _name2 in self.__feat_artist_names:
                        if _name2.matchs(str(_name1)):
                            _ret = 1
                            break
        return _ret

    def __cmp__(self, p_artist):
        """Compare with another artist."""
        _ret = 1
        if p_artist is not None:
#            _ret = cmp(self.__name, p_artist.__name)
            _ret = cmp(self.__artist_names, p_artist.__artist_names)
            if _ret == 0:
                _ret = cmp(self.__feat_artist_names, p_artist.__feat_artist_names)
        return _ret

#    def get_formatted_string(self, p_names_sep = NAMES_SEP, p_feat_sep = FEAT_SEP):
#        """Return a formatted representation as a string."""
#        _ret = ''
#        for _name in self.__artist_names[:-1]:
#            _ret += str(_name) + p_names_sep
#        _ret += str(self.__artist_names[-1])
#        if len(self.__feat_artist_names) > 0:
#            _ret += p_feat_sep
#            for _name in self.__feat_artist_names[:-1]:
#                _ret += str(_name) + p_names_sep
#            _ret += str(self.__feat_artist_names[-1])
#        return _ret        

    def __str__(self):
        """Return a representation as a string."""
        _ret = ""
        for _name in self.__artist_names[:-1]:
            _ret += str(_name) + NAMES_SEP
        _ret += str(self.__artist_names[-1])
        if len(self.__feat_artist_names) > 0:
            _ret += FEAT_SEP
            for _name in self.__feat_artist_names[:-1]:
                _ret = str(_name) + NAMES_SEP
            _ret = str(self.__feat_artist_names[-1])
        """
        _ret = self.get_formatted_string()
        """
        return _ret        

    def write_xml(self, p_output):
        """Write a represetation as a XML string."""
        _output = p_output
        _output.write_begin_tag(ARTIST_TAG)
        for _name in self.__artist_names:
            _output.write_tags_with_data(ARTIST_NAME_TAG, str(_name))
        for _name in self.__feat_artist_names:
            _output.write_tags_with_data(FEAT_ARTIST_NAME_TAG, str(_name))
        _output.write_end_tag(ARTIST_TAG)
        return _output


def test(p_name):
    import sys
    from util.XMLOutput import XMLOutput
    print "__init__(", p_name, ")"
    _a = Artist(p_name)
    print "__str__() :", _a
    print "write_xml()"
    _output = XMLOutput(sys.stdout)    
    _a.write_xml(_output)
    return _a

if __name__ == "__main__":
    _a1 = test("Shania Twain")
    _a2 = test("Organiz'")
    print "cmp(_a1, _a2) :", cmp(_a1, _a2)
    print "_a1.matchs(_a2) :", _a1.matchs(_a2)
