# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from Artist import Artist
from util.String import String

DISC_TAG = 'disc'

DISC_TITLE_TAG = 'title'

DISC_SEP = ' - '

class Disc:
    """A musical disc.

    Warning ! For a reason of performance, this class is not synchronized.
    Don't apply its modificators to a shared instance.
    """

    __title = String("")

    __artist = None

    def __init__(self, p_title, p_artist = None):
        """Construct a disc."""
        self.__title = String(p_title)
        self.__artist = p_artist

    def get_title(self):
        """Return the title."""
        return self.__title.data

    def set_title(self, p_title):
        """Set the given title."""
        self.__title = String(p_title)

    def has_artist(self):
        """Determine if the disc has an artist."""
        return (self.__artist is not None)

    def get_artist(self):
        """Return the artist."""
        return self.__artist

    def set_artist(self, p_artist):
        """Set the given artist."""
        self.__artist = p_artist

    def set_no_artist(self):
        """Set that the disc has no artist."""
        self.__artist = None

    def clone(self):
        """Return a clone of the disc."""
        _new_artist = None
        if self.has_artist():
            _new_artist = self.__artist.clone()
        _disc = Disc(self.get_title(), _new_artist)
        return _disc

    def matchs(self, p_disc):
        """Determine if another disc matchs the disc."""
        _ret = 1
        if p_disc is not None:
            if not self.__title.matchs(p_disc.get_title()):
                _ret = 0
            elif self.has_artist() and not p_disc.has_artist():
                _ret = 1
            elif not self.has_artist() and p_disc.has_artist():
                _ret = 0
            elif not self.has_artist() and not p_disc.has_artist():
                _ret = 1
            elif self.has_artist() and p_disc.has_artist():
                _ret = self.__artist.matchs(p_disc.get_artist())
        return _ret

    def __cmp__(self, p_disc):
        """Compare with another disc."""
        _ret = 1
        if p_disc is not None:
            _ret = cmp(self.__title, p_disc.__title)
            if _ret == 0:
                _ret = cmp(self.__artist, p_disc.__artist)
        return _ret

    def __str__(self):
        """Give a representation as a string."""
        _ret = self.get_title()
        if self.has_artist():
            _ret += DISC_SEP
            _ret += str(self.get_artist())
        return _ret

    def write_xml(self, p_output):
        """Write a represetation as a XML string."""
        _output = p_output
        _output.write_begin_tag(DISC_TAG)
        _output.write_tags_with_data(DISC_TITLE_TAG, self.get_title())
        if self.has_artist():
            _output = self.__artist.write_xml(_output)
        _output.write_end_tag(DISC_TAG)
        return _output


if __name__ == "__main__":
    import sys
    from util.XMLOutput import XMLOutput
    _artist1 = Artist("Author")
    _artist2 = Artist("Author")
    _disc1 = Disc("Title", _artist1)
    _disc2 = Disc("Title", _artist2)
    _disc3 = Disc("Title")
    _disc4 = Disc("Title")
    print "_disc1.has_artist() :", _disc1.has_artist()
    print "_disc1 :", _disc1
    print "_disc2 :", _disc2
    print "_disc3 :", _disc3
    print "_disc4 :", _disc4
    print "_disc1.write_xml() :"
    _output = XMLOutput(sys.stdout)    
    _disc1.write_xml(_output)
    print "_disc1 == _disc2 :", _disc1 == _disc2
    print "_disc3 == _disc4 :", _disc3 == _disc4
    print "_disc1 == _disc3 :", _disc1 == _disc3
    print "_disc3 == _disc1 :", _disc3 == _disc1
    print "_disc1.matchs(_disc2) :", _disc1.matchs(_disc2)
    print "_disc3.matchs(_disc4) :", _disc3.matchs(_disc4)
    print "_disc1.matchs(_disc3) :", _disc1.matchs(_disc3)
    print "_disc3.matchs(_disc1) :", _disc3.matchs(_disc1)
