# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from Task import Task
from Process import is_in_path
from Process import ToolNotFoundException
from Process import Process
from util.Directory import DirectoryNotFoundException
from util.Directory import DirectoryNotWritableException
from util.File import FileNotFoundException
from util.File import FileNotReadableException
from util.File import File

ENCODING_TOOL = "oggenc"

ENCODING_TOOL_EXISTS = is_in_path(ENCODING_TOOL)


class Encoding(Task):
    """A task which encodes a wave file into an ogg Vorbis file."""

    __src_file_path = ""

    __dest_file_path = ""

    __process = None

    def __init__(self, p_src_path, p_dest_path):
        Task.__init__(self)        
        self.__src_file_path = p_src_path
        self.__dest_file_path = p_dest_path
        self.__process = None

    def get_src_file_path(self):
        return self.__src_file_path

    def get_dest_file_path(self):
        return self.__dest_file_path


    def start(self):
        """Encode a wave file into an Ogg Vorbis file.

        Raise ToolNotFoundException if a tool is not available.
        Raise FileNotFoundException if the file does not exist.
        Raise FileNotReadableException if the file is not readable.
        Raise DirectoryNotFoundException if the temporary directory does not exist.
        Raise DirectoryNotWritableException if the temporary file cannot be
        created."""
        # verify that the encode tool is available
        if not ENCODING_TOOL_EXISTS:
            raise ToolNotFoundException, ENCODING_TOOL
        _file = File(self.get_src_file_path())
        # verify that the file exists
        if not _file.exists():
            raise FileNotFoundException, _file.get_path()
        # verify that the file is readable
        if not _file.is_readable():
            raise FileNotReadableException, _file.get_path()
        # get a temporary file path
        _dest_file = File(self.get_dest_file_path())
        _dir = _dest_file.get_parent_directory()
        # verify that the temporary directory does exist
        if not _dir.exists():
            raise DirectoryNotFoundException, _dir.get_path()
        # verify if the temporary directory is writable
        if not _dir.is_writable():
            raise DirectoryNotWritableException, _dir.get_path()
        # launch process
        _cmd = ENCODING_TOOL + " -q -r \"" + self.get_src_file_path() + "\" -o \"" + self.get_dest_file_path() + "\""
        self.__process = Process(_cmd, Process.IO_LOG_FILE)
        self.__process.wait()


    def stop(self):
        if self.__process is not None:
            self.__process.kill()


if __name__ == '__main__':
    import sys
    if not ENCODING_TOOL_EXISTS:
        print 'Required tool : %s' % ENCODING_TOOL
    elif len(sys.argv) != 3:
        print 'Usage : %s %s src_file dest_file' % (sys.executable, sys.argv[0])
    else:
        _task = Encoding(sys.argv[1], sys.argv[2])
        print 'Encoding the file :'
        print '\t%s' % _task.get_src_file_path()
        print 'into the file :'
        print '\t%s' % _task.get_dest_file_path()
        print 'Please wait...'
        _task.start()
        
