# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import os
import threading

from util.Directory import Directory

from Selection import Selection
from SongFile import SongFile

LIBRARY_TAG = 'library'

ROOT_PATH_ATT = 'root'

FILE_TAG = 'file'

FILE_NAME_ATT = 'name'

class Library:
    """An Ogg Vorbis library."""

    __root_dir = None

    __songs = {}

    __lock = None

    def __init__(self, p_root_path = os.getcwd()):
        """Construct an Ogg Vorbis library."""
        self.__root_dir = Directory(p_root_path)
        self.__songs = {}
        self.__lock = threading.Lock()
        
    def set_root_path(self, p_root_path):
        self.__lock.acquire()
        self.__root_dir = Directory(p_root_path)
        self.__lock.release()
        
    def get_root_path(self):
        self.__lock.acquire()
        _ret = self.__root_dir.get_path()
        self.__lock.release()
        return _ret

    def has_song(self, p_file_name):
        """Determine if the file name belongs to the library."""
        self.__lock.acquire()
        _ret = self.__songs.has_key(p_file_name)
        self.__lock.release()
        return _ret

    def get_song(self, p_file_name):
        """Return the song corresponding of the given file name.

        Raise KeyError if the file name is not in the library."""
        self.__lock.acquire()
        try:
            _ret = self.__songs[p_file_name]
        finally:
            self.__lock.release()
        return _ret

    def add_song(self, p_file_path, p_song):
        """Add a song in the library."""
        """
        _song = p_song
        # verifies all singers
        _singers = _song.get_singers()
        _new_singers = self.__verify_artists(_singers)
        _song.set_singers(_new_singers)
        # verifies all "featurers"
        _feat = _song.get_feat()
        _new_feat = self.__verify_artists(_feat)
        _song.set_feat(_new_feat)
        # verifies the tune
        _tune = _song.get_tune()
        _tune = self.__verify_tune(_tune)
        _song.set_tune(_tune)
        # verifies the disc
        _disc = _song.get_disc()
#        print _disc, " <---> ",
        _disc = self.__verify_disc(_disc)
#        print _disc
        _song.set_disc(_disc)
        # if the name of the artist is found in the path name,
        # the disc is an album
        _path = os.path.dirname(p_file_path)
        _disc = _song.get_disc()
        _singers = _song.get_singers()
        if len(_singers) > 0:
            _artist = _singers[0]
            if _artist is not None and matchs(_path, _artist.get_name()):
                _disc.set_artist(_artist)
        # adds the song to the library
        """
        self.__lock.acquire()
        self.__songs[p_file_path] = p_song
        self.__lock.release()

    def remove_song(self, p_file_path):
        """Remove a song from the library."""
        self.__lock.acquire()
        try:
            del self.__songs[p_file_path]
        finally:
            self.__lock.release()

    def remove_all_songs(self):
        """Remove all songs of the library."""
        self.__lock.acquire()
        self.__songs.clear()
        self.__lock.release()

    def __get_all_names(self):
        self.__lock.acquire()
        _ret = self.__songs.keys()
        self.__lock.release()
        return _ret

    def get_selection(self):
        """Get a selection of all music files."""
        _sel = Selection()
        _names = self.__get_all_names()
        for _name in _names:
            _song = self.get_song(_name)
            _file = SongFile(_name, _song)
            _sel.append(_file)
        _sel.sort()
        return _sel

    def __str__(self):
        """Return a representation as a string."""
        _ret = ''
        _cpt = 0
        _ret += 'Library with root path ' + self.get_root_path() + ' : \n'
        _names = self.__get_all_names()
        for _name in _names:
            _cpt += 1
            _ret += '\t' + str(_cpt) + '\t--> ' + _name + '\t--> ' + str(self.get_song(_name)) + '\n'
        return _ret

