# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from util.File import File
from util.XMLOutput import XMLOutput

from Library import LIBRARY_TAG
from Library import ROOT_PATH_ATT
from Library import FILE_TAG
from Library import FILE_NAME_ATT

from Task import Task


class LibrarySaving(Task):
    """An task which saves a library into a XML file."""

    __library = None

    __file_path = ''

    def __init__(self, p_library, p_file_path):
        Task.__init__(self)
        self.__library = p_library
        self.__file_path = p_file_path

    def get_file_path(self):
        return self.__file_path


    def start(self):
        """Write XML string in the given file object."""
        _file = File(self.get_file_path())
        # open the file
        _f = _file.open_to_write()
        # write the document head
        _output = XMLOutput(_f)
        _output.write_begin_root_tag(LIBRARY_TAG, {ROOT_PATH_ATT:self.__library.get_root_path()})
        # get the library as a selection
        _sel = self.__library.get_selection()
        # write all library songs
        for _song_file in _sel:
            _output.write_begin_tag(FILE_TAG, {FILE_NAME_ATT:_song_file.get_path()})
            _song = _song_file.get_song()
            if _song is not None:
                _output = _song.write_xml(_output)
            _output.write_end_tag(FILE_TAG)
        # write the document bottom
        _output.write_end_root_tag(LIBRARY_TAG)
        # close the file
        _f.close()
            


if __name__ == '__main__':
    from Library import Library
    from Song import get_empty_song
    # create a library
    _lib = Library()
    # add some songs
    _song = get_empty_song()
    _lib.add_song('first_file.ogg', _song)
    _lib.add_song('second_file.ogg', _song)
    # save the library
    _saving = LibrarySaving(_lib, 'library.xml')
    _saving.start()
