# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from util.Directory import Directory
from util.File import File

from Task import Task
from Converting import CONVERTING_TOOL
from Converting import CONVERTING_TOOL_EXISTS
from Converting import Converting
from WaveImporting import get_wave_importing_tools
from WaveImporting import can_import_wave
from WaveImporting import WaveImporting


def get_mp3_importing_tools():
    return [CONVERTING_TOOL] + get_wave_importing_tools()


def can_import_mp3():
    return (CONVERTING_TOOL_EXISTS and can_import_wave())


class MP3Importing(Task):
    """A task which imports a MP3 file into the library."""

    __converting = None

    __wave_importing = None

    def __init__(self, p_manager, p_library, p_src_file_path, p_song):
        Task.__init__(self)
        # get a temporary file path
        _dir = Directory(p_manager.get_tmp_path())
        _inode = _dir.get_inode('song.wav')
        # create a converting task
        self.__converting = Converting(p_src_file_path, _inode.get_path())
        # create a wave importing task
        self.__wave_importing = WaveImporting(p_manager, p_library, _inode.get_path(), p_song)

    def get_src_file_path(self):
        return self.__converting.get_src_file_path()

    def get_dest_file_path(self):
        return self.__wave_importing.get_dest_file_path()

    def get_song(self):
        return self.__wave_importing.get_song()

    def start(self):
        """Convert a MP3 file into an Ogg Vorbis file and add it to the library.

        Raise ToolNotFoundException if a tool is not available.
        Raise FileNotFoundException if the file does not exist.
        Raise FileNotReadableException if the file is not readable.
        Raise FileNotMP3Exception.
        Raise DirectoryNotFoundException if the temporary directory does not exist.
        Raise DirectoryNotWritableException if the temporary file cannot be
        created.
        Raise ProcessException if a process stops irregularly."""
        # convert the MP3 file
        self.__converting.start()
        # import the wave result file to the library
        self.__wave_importing.start()
        # remove the temporary file
        _file = File(self.__wave_importing.get_src_file_path())
        _file.remove()

    def stop(self):
        Task.stop(self)
        # convert the MP3 file in a wave file
        if self.__converting is not None:
            self.__converting.stop()
        # import the wave file
        if self.__wave_importing is not None:
            self.__wave_importing.stop()


if __name__ == '__main__':
    import sys
    import string
    from Manager import Manager
    from Library import Library
    from Song import get_complete_song
    if not can_import_mp3():
        print 'Required tools : %s' % string.join(get_mp3_importing_tools())
    elif len(sys.argv) != 5:
        print 'Usage : %s %s file artist title album' % (sys.executable, sys.argv[0])
    else:
        _manager = Manager()
        _lib = Library()
        _file_path = sys.argv[1]
        _song = get_complete_song(sys.argv[2], sys.argv[3], sys.argv[4])
        _task = MP3Importing(_manager, _lib, _file_path, _song)
        print 'Importing the file :'
        print '\t%s' % _task.get_src_file_path()
        print 'to the file :'
        print '\t%s' % _task.get_dest_file_path()
        print 'with song :'
        print '\t%s' % _task.get_song()
        print 'Please wait...'
        _task.start()
        print _lib
