# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from util.File import File

from Task import Task
from PathConfig import FORMAT
from PathConfig import DISC_FORMAT
from PathConfig import ARTISTS_SEP
from PathConfig import SPACE_SUB
from PathConfig import LOWERCASE
from PathConfig import PathConfig


class PathConfigLoading(Task):
    """A task which loads a path configuration from a file."""

#    __path_config = None

    __manager = None

    __file_path = ''

    def __init__(self, p_manager, p_file_path):
        Task.__init__(self)
        self.__manager = p_manager
#        self.__path_config = p_path_config
        self.__file_path = p_file_path

    def get_file_path(self):
        return self.__file_path


    def start(self):
        """Load the path configuration from the file.

        Raise FileNotFoundException if the file does not exist.
        Raise FileNotReadableException if the file is not readable.
        Raise DirectoryNotFoundException if the root directory does not exist.
        Raise DirectoryNotAccessibleException if the root directory cannot be
        accessed."""
        _path_config = PathConfig()
        _file = File(self.__file_path)
        _f = _file.open_to_read()
        _line = _f.readline()
        while _line:
            _line = _line[:-1]
            # get the tag and its value from the line
            _tag = ''
            _value = ''
            _tab = _line.split('=', 1)
            if len(_tab) >= 1:
                _tag = _tab[0]
            if len(_tab) >= 2:
                _value = _tab[1]
            # set the attribute
#            if _tag == ROOT_PATH:
#                _path_config.set_root_path(_value)
#            elif _tag == TMP_PATH:
#                _path_config.set_tmp_path(_value)
            if _tag == FORMAT:
                _path_config.set_format(_value)
            elif _tag == DISC_FORMAT:
                _path_config.set_disc_format(_value)
            elif _tag == ARTISTS_SEP:
                _path_config.set_artists_sep(_value)
            elif _tag == SPACE_SUB:
                _path_config.set_space_substitute(_value)
            elif _tag == LOWERCASE:
                _path_config.set_lowercasing(int(_value))
            # read the next line
            _line = _f.readline()
        # close file
        _f.close()
        self.__manager.set_path_config(_path_config)


if __name__ == '__main__':
    _config = PathConfig()
    _loading = PathConfigLoading(_config, '/tmp/test/path.conf')
    _loading.start()
    print _config
