# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from util.Directory import DirectoryNotFoundException
from util.Directory import Directory

from Task import Task
from SongFile import SongFile
from PlayList import PlayList


class PlayListsLoading(Task):
    """A task which loads all playlists from a directory."""

    __play_lists_set = None

    __library = None

    __dir_path = ''

    def __init__(self, p_play_lists_set, p_library, p_dir_path):
        Task.__init__(self)
        self.__play_lists_set = p_play_lists_set
        self.__library = p_library
        self.__dir_path = p_dir_path

    def get_dir_path(self):
        return self.__dir_path


    def start(self):
        """Load the playlists from files :
        '$HOME/.maestro/playlists/pl-n-my_playlist_title'
        where n is the rank of the playlist.

        Raise DirectoryNotFoundException if the directory does not exist.
        Raise DirectoryNotReadableException if the directory is not readable.
        Raise FileNotReadableException if a file is not readable."""
        # clear the play list set
        self.__play_lists_set.clear()
        # verify the existence of the directory
        _dir = Directory(self.get_dir_path())
        if not _dir.exists():
            raise DirectoryNotFoundException, _dir.get_path()
        _list = _dir.list_files()
        _list.sort()
        for _entry in _list:
            _name = _entry.get_name()
            try:
                (_suf, _num, _title) = _name.split('-', 2)
            except ValueError:
                pass
            else:
                _title = _title.replace('_', ' ')
                # create a playlist
                _pl = PlayList(_title)
                # open the file
                _file = open(_entry.get_path())
                # read the file and construct the playlist
                _name = _file.readline()
                _name = _name.strip()
                while _name:
                    if self.__library.has_song(_name):
                        _song = self.__library.get_song(_name)
                    else:
                        _song = None
                    _music_file = SongFile(_name, _song)
                    _pl.append(_music_file)
                    _name = _file.readline()
                    _name = _name.strip()
                # close the file
                _file.close()
                # add the play list
                self.__play_lists_set.append(_pl)


if __name__ == '__main__':
    from PlayListsSet import PlayListsSet
    from Library import Library
    _pls = PlayListsSet()
    _lib = Library()
    _loading = PlayListsLoading(_pls, _lib, '/tmp')
    _loading.start()
