# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from util.UserException import UserException
from util.Directory import Directory
from util.File import File

from Task import Task

class PlayListsSavingException(UserException):
    """An exception raised when somme play lists cannot be saved."""

    __errors = []

    def __init__(self):
        UserException.__init__(self)
        self.__errors = []

    def add_error(self, p_path, p_error):
        """Add a couple (path, error)."""
        _c = (p_path, p_error)
        self.__errors.append(_c)

    def is_empty(self):
        """Determine if no error occured while the play lists saving."""
        return len(self.__errors) == 0

    def get_errors_number(self):
        """Return the number of errors."""
        return len(self.__errors)

    def get_error(self, p_index):
        """Return the couple (path, error) which cannot be saved.""" 
        return self.__errors[p_index]
        

class PlayListsSaving(Task):
    """A task which saves all playlists in a directory."""

    __play_lists_set = None

    __dir_path = ''

    def __init__(self, p_play_lists_set, p_dir_path):
        Task.__init__(self)
        self.__play_lists_set = p_play_lists_set
        self.__dir_path = p_dir_path

    def get_dir_path(self):
        return self.__dir_path

    def start(self):
        """Write all play lists in file like :
        'dir/pl-n-my_playlist_title'
        where n is the rank of the playlist.

        Create the directory if it does not exist.
        Raise DirectoryNotReadableException if the directory is not readable.
        Raise DirectoryNotWritableException if the directory is not writable.
        Raise PlayListsSavingException if plays lists files cannot be removed or created."""
        _plse = PlayListsSavingException()
        # create a directory
        _dir = Directory(self.get_dir_path())
        if not _dir.exists():
            _dir.create()
        # remove all play lists
        _list = _dir.list_files()
        for _entry in _list:
            _file = File(_entry.get_path())
            try:
                _file.remove()
            except IOError, _exc:
                _plse.add_error(_file.get_path(), _exc)
            except UserException, _exc:
                _plse.add_error(_file.get_path(), _exc)
            """
            _name = _entry.get_name()
            (_suf, _num, _title) = _name.split('-', 3)
            if not self.has_play_list(_title):
                _entry.remove()
            """
        # write all play list files
        _len = self.__play_lists_set.get_length()
        for _num in range(_len):
            _pl = self.__play_lists_set[_num]
            _title = _pl.get_title()
            _name = 'pl-' + str(_num) + '-' + _title.replace(' ', '_')
            _file_path = _dir.get_file_path(_name)
            try:
                _f = open(_file_path, 'w')
                _f.write(str(_pl))
                _f.close()
            except IOError, _exc:
                _plse.add_error(_file_path, _exc)
        # if at least one error occurs, raise the exception
        if not _plse.is_empty():
            raise _plse


if __name__ == '__main__':
    from PlayListsSet import PlayListsSet
    _pls = PlayListsSet()
    _saving = PlayListsSaving(_pls, '/tmp')
    _saving.start()
