# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import threading

from util.QueuedThread import QueuedThread
from Process import ProcessException
from SongPlaying import SongPlaying
from util.Inode import InodeException
from Selection import Selection


class PlayThread(QueuedThread):
    "A thread that plays songs of a queue."

    __task = None

    __looping = 0

    __looping_lock = None

    def __init__(self):
        _sel = Selection()
        QueuedThread.__init__(self, "Playing queue", _sel)
        self.__task = None
        self.__looping = 0
        self.__looping_lock = threading.Lock()

    ###########################################################################
    # ACCESSORS
    ###########################################################################

    def get_current_file(self):
        """Return the music file which is currently played."""
        return self.get_current_item()

    def get_looping(self):
        """Determine if the looping is activated."""
        self.__looping_lock.acquire()
        _ret = self.__looping
        self.__looping_lock.release()
        return _ret


    ###########################################################################
    # MODIFCATORS
    ###########################################################################

    def set_looping(self, p_bool):
        """Set the looping."""
        self.__looping_lock.acquire()
        self.__looping = p_bool
        self.__looping_lock.release()

    def play_next(self):
        """Play next music file."""
        self.stop()
        self.restart()

    ###########################################################################
    # MUSIC FILE HANDLING
    ###########################################################################

    def stop_item_handling(self):
        """Stop the current task."""
        if self.__task is not None:
            self.__task.stop()

    def handle_item(self, p_file):
        """Play the given music file."""
        self.__task = SongPlaying(p_file.get_path())
        try:
            self.__task.start()
        except ProcessException:
            pass
        except InodeException:
            pass
        # if looping is enable, add the music file to the end of the queue
        if self.get_looping():
            self.get_queue().append(p_file)
