# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from Task import Task
from Process import is_in_path
from Process import ToolNotFoundException
from Process import Process
from util.Directory import DirectoryNotFoundException
from util.Directory import DirectoryNotWritableException
from util.File import File

RIPPING_TOOL = "cdparanoia"

RIPPING_TOOL_EXISTS = is_in_path(RIPPING_TOOL)


class Ripping(Task):
    """A task which rips a track."""

    __track = 0

    __dest_file_path = ""

    __process = None

    def __init__(self, p_track, p_dest_path):
        Task.__init__(self)        
        self.__track = p_track
        self.__dest_file_path = p_dest_path
        self.__process = None

    def get_track(self):
        return self.__track

    def get_dest_file_path(self):
        return self.__dest_file_path


    def start(self):
        """Rip a CDDA track and save it as a wave file.

        Raise ToolNotFoundException if the tool is not available.
        Raise DirectoryNotFoundException if the temporary directory does not exist.
        Raise DirectoryNotWritableException if the temporary file cannot be
        created."""
        # verify that the rip tool is available
        if not RIPPING_TOOL_EXISTS:
            raise ToolNotFoundException, RIPPING_TOOL
        _dest_file = File(self.get_dest_file_path())
        _dir = _dest_file.get_parent_directory()
        # verify that the temporary directory does exist
        if not _dir.exists():
            raise DirectoryNotFoundException, _dir.get_path()
        # verify if the temporary directory is writable
        if not _dir.is_writable():
            raise DirectoryNotWritableException, _dir.get_path()
        # launch process
        _cmd = RIPPING_TOOL + " -q -s " + str(self.get_track()) + " \"" + self.get_dest_file_path() + "\""
        self.__process = Process(_cmd, Process.IO_LOG_FILE)
        self.__process.wait()


    def stop(self):
        if self.__process is not None:
            self.__process.kill()


if __name__ == '__main__':
    import sys
    if not RIPPING_TOOL_EXISTS:
        print 'Required tool : %s' % RIPPING_TOOL
    elif len(sys.argv) != 3:
        print 'Usage : %s %s track dest_file' % (sys.executable, sys.argv[0])
    else:
        _task = Ripping(sys.argv[1], sys.argv[2])
        print 'Ripping the track :'
        print '\t%s' % _task.get_track()
        print 'into the file :'
        print '\t%s' % _task.get_dest_file_path()
        print 'Please wait...'
        _task.start()

