# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from util.File import File

from Task import Task


class RootRemoving(Task):
    """A task which removes a file from the root directory.
    If the destination file path is given, the file will be copied."""

    __src_file_path = ''

    __dest_file_path = ''

    def __init__(self, p_src_path, p_dest_path = ''):
        Task.__init__(self)        
        self.__src_file_path = p_src_path
        self.__dest_file_path = p_dest_path

    def get_src_file_path(self):
        return self.__src_file_path

    def get_dest_file_path(self):
        return self.__dest_file_path

    def start(self):
        """Remove the file from the root directory.

        Raise FileNotFoundException if the file does not exist.
        Raise FileNotReadableException if the file is not readable.
        Raise DirectoryNotWritableException if the directory cannot be
        created in the root directory or if the file cannot be copied in the
        directory."""
        _src_file = File(self.__src_file_path)
        if self.__dest_file_path:
            _dest_file = File(self.__dest_file_path)
            # move the given file in a temporary directory
            _src_file.move(_dest_file)
        else:
            # remove the file
            _src_file.remove()
        # delete directory if necessary
        _dir = _src_file.get_parent_directory()
        _pdir = _dir.get_parent()
        if _dir.is_empty() and _pdir.is_writable():
            _dir.delete()



if __name__ == '__main__':
    import sys
    _len = len(sys.argv)
    if _len != 2 and _len != 3:
        print 'Usage : %s %s src_file [dest_file]' % (sys.executable, sys.argv[0])
    else:
        if _len == 2:
            _task = RootRemoving(sys.argv[1])
            print 'Removing the file :'
        else:
            _task = RootRemoving(sys.argv[1], sys.argv[2])
            print 'Moving the file :'
        print '\t%s' % _task.get_src_file_path()
        if _len == 3:
            print 'to the file :'
            print '\t%s' % _task.get_dest_file_path()
        print 'Please wait...'
        _task.start()
    
