# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#import UserList
import random

from util.SyncList import SyncList
from util.String import String
from util.NumberString import NumberString


class Selection(SyncList):
    """A selection of Ogg Vorbis files."""

#    def prepend(self, p_file):
#        "Add the given file at the beginning."""
#        self.insert(0, p_file)

#    def delete(self, p_index):
#        """Remove the file at the given index.
#
#        Raise IndexError if the index is out of bound."""
#        del self[p_index]

#    def clear(self):
#        """Empty the selection."""
#        while not self.is_empty():
#            self.delete(0)

    def random(self):
        """Randomize the order."""
        _len = self.get_length() - 1
        while _len > 0:
            _index = random.randrange(0, _len)
            _tmp = self[_index]
            self[_index] = self[_len]
            self[_len] = _tmp
            _len -= 1
        
#    def pop_first(self):
#        """Return the head file."""
#        _ret = ""
#        if not self.is_empty():
#            _ret = self.pop(0)
#        return _ret

#    def get_length(self):
#        """Return the length."""
#        return len(self)

#    def is_empty(self):
#        """Determine if the selection is empty."""
#        return (self.get_length() == 0)

    def index_by_path(self, p_path):
        """Return the index of the first file which has the given path.

        Raise IndexError if there is no file with the given path."""
        _index = 0
        while not self[_index].has_path(p_path):
            _index += 1
        return _index
        
    def update(self, p_path, p_m_file):
        """Replace the first file which has the given path by the given file.

        Raise IndexError if there is no file with the given file."""
        _index = self.index_by_path(p_path)
        self[_index] = p_m_file

    def remove_by_path(self, p_path):
        """Remove the file which has the given path.

        Raise IndexError if there is no file with the given file."""
        _index = self.index_by_path(p_path)
        self.delete(_index)

    def search_song(self, p_song):
        """Search the given song and return a new selection."""
        _ret = Selection()
        for _file in self:
            _song = _file.get_song()
            if _song is not None and _song.matchs(p_song):
                _ret.append(_file)
        return _ret

    def search_artist(self, p_artist = None):
        """Search for an artist."""
        _ret = []
        # browses all songs to match an artist
        for _file in self:
            _song = _file.get_song()
            """
            # gets the artists of the song
            _singers = _song.get_singers()
            _feat = _song.get_feat()
            for _artist in _singers + _feat:
                # if the two artists are equal,
                if _artist.matchs(p_artist):
                    # and if the disc have never be found
                    if _artist not in _ret:
                        # the artist is appended
                        _ret.append(_artist)
            """
            # gets the artists of the song
            _artist = _song.get_artist()
            # if the two artists are equal,
            if _artist is not None and _artist.matchs(p_artist):
                # and if the artist has never be found
                if _artist not in _ret:
                    # the artist is appended
                    _ret.append(_artist)
        return _ret

    def search_disc(self, p_disc = None):
        """Search for a disc."""
        _ret = []
        # browses all songs to match a disc
        for _file in self:
            _song = _file.get_song()
            # gets the disc of the song
            _disc = _song.get_disc()
            # if the two discs are equal,
            if _disc.matchs(p_disc):
                # and if the disc has never be found
                if _disc not in _ret:
                    # the disc is appended
                    _ret.append(_disc)
        return _ret

    def search_genre(self, p_genre = ''):
        """Search for a genre."""
        _ret = []
        # browses all songs to match a genre
        for _file in self:
            _song = _file.get_song()
            # gets the genre of the song
            _genre = _song.get_genre()
            _genre_str = String(_genre)
            # if the two genres are equal,
            if _genre_str.matchs(p_genre):
                # and if the genre has never be found
                if _genre not in _ret:
                    # the genre is appended
                    _ret.append(_genre)
        return _ret

    def search_tune(self, p_tune = None):
        """Search a tune."""
        _ret = []
        # browses all songs to match a tune
        for _file in self:
            _song = _file.get_song()
            # gets the tune of the song
            _tune = _song.get_tune()
            # if the two tunes are equal,
            if _tune.matchs(p_tune):
                # and if the tune have never be found
                if _tune not in _ret:
                    # the disc is appended
                    _ret.append(_tune)
        return _ret

    def __compare(self, p_file1, p_file2):
        """Compare two files."""
        _song1 = p_file1.get_song()
        _song2 = p_file2.get_song()
        if _song1 is None and _song2 is None:
            _ret = 0
        elif _song1 is None and _song2 is not None:
            _ret = 1
        elif _song1 is not None and _song2 is None:
            _ret = -1
        else:
            _ret = 2
        return _ret

    def __compare_title(self, p_file1, p_file2):
        """Compare titles of two files."""
        _ret = self.__compare(p_file1, p_file2)
        if _ret > 1:
            _ret = cmp(p_file1.get_song().get_tune().get_title(),
                       p_file2.get_song().get_tune().get_title())
        return _ret

    def sort_by_title(self):
        """Sort by title."""
        self.sort(self.__compare_title)

    def __compare_year(self, p_file1, p_file2):
        """Compare years of two files."""
        _ret = self.__compare(p_file1, p_file2)
        if _ret > 1:
            _ns1 = NumberString(p_file1.get_song().get_tune().get_year())
            _ns2 = NumberString(p_file2.get_song().get_tune().get_year())
            _ret = cmp(_ns1, _ns2)
#            _ret = compare_str_as_int(p_file1.get_song().get_tune().get_year(),
#                                      p_file2.get_song().get_tune().get_year())
        return _ret

    def sort_by_year(self):
        """Sort by year."""
        self.sort(self.__compare_year)

    def __compare_artist(self, p_file1, p_file2):
        """Compare artists of two files."""
        _ret = self.__compare(p_file1, p_file2)
        if _ret > 1:
            _artist1 = p_file1.get_song().get_artist()
            _artist2 = p_file2.get_song().get_artist()
            _ret = cmp(_artist1, _artist2)
            """
            _singers1 = p_file1.get_song().get_singers()
            _singers2 = p_file2.get_song().get_singers()
            _ret = cmp(_singers1, _singers2)
            """
        return _ret

    def sort_by_artist(self):
        """Sort by artists."""
        self.sort(self.__compare_artist)

    def __compare_disc(self, p_file1, p_file2):
        """Compare discs of two files."""
        _ret = self.__compare(p_file1, p_file2)
        if _ret > 1:
            _ret = cmp(p_file1.get_song().get_disc(), p_file2.get_song().get_disc())
        return _ret

    def sort_by_disc(self):
        """Sort by disc."""
        self.sort(self.__compare_disc)

    def __compare_track(self, p_file1, p_file2):
        """Compare tracks of two files."""
        _ret = self.__compare(p_file1, p_file2)
        if _ret > 1:
            _ns1 = NumberString(p_file1.get_song().get_track())
            _ns2 = NumberString(p_file2.get_song().get_track())
            _ret = cmp(_ns1, _ns2)
#            _ret = compare_str_as_int(p_file1.get_song().get_track(),
#                                      p_file2.get_song().get_track())
        return _ret

    def sort_by_track(self):
        """Sort by track."""
        self.sort(self.__compare_track)

    def __compare_genre(self, p_file1, p_file2):
        """Compare genres of two files."""
        _ret = self.__compare(p_file1, p_file2)
        if _ret > 1:
            _ret = cmp(p_file1.get_song().get_genre(), p_file2.get_song().get_genre())
        return _ret

    def sort_by_genre(self):
        """Sort by genre."""
        self.sort(self.__compare_genre)
