# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from Task import Task
from TagsWriting import TAGS_WRITING_TOOL
from TagsWriting import TAGS_WRITING_TOOL_EXISTS
from TagsWriting import TagsWriting
from TagsSet import TagsSet


def get_song_adding_tools():
    return [TAGS_WRITING_TOOL]


def can_add_song():
    return TAGS_WRITING_TOOL_EXISTS


class SongAdding(Task):
    """A task which adds a song file into the root directory
    and into the library."""

    __library = None

    __song = None

    __tags_writing = None

    def __init__(self, p_manager, p_library, p_src_file_path, p_song):
        Task.__init__(self)
        _set = TagsSet(p_song)
        _dest_file_path = p_manager.get_path_config().get_correct_file_path(p_song)
        self.__library = p_library
        self.__song = p_song
        self.__tags_writing = TagsWriting(p_src_file_path, _dest_file_path, _set)

    def get_src_file_path(self):
        return self.__tags_writing.get_src_file_path()

    def get_dest_file_path(self):
        return self.__tags_writing.get_dest_file_path()

    def get_song(self):
        return self.__song.clone()


    def start(self):
        """Add the file in the root directory and in the library
        and return its new path.

        Raise ToolNotFoundException if the tool for writing tags is not available.
        Raise FileNotFoundException if the file does not exist.
        Raise FileNotReadableException if the file is not readable.
        Raise FileNotOggVorbisFileException if the file is not an Ogg Vorbis file.
        Raise DirectoryNotWritableException if the directory cannot be
        created in the root directory or if the file cannot be copied in the
        directory.
        Raise ProcessException if a process stops irregularly."""
        # copy the file and write tags in the same time
        self.__tags_writing.start()
        # add the song in the library
        self.__library.add_song(self.get_dest_file_path(), self.__song)
        


if __name__ == '__main__':
    import sys
    import string
    from Manager import Manager
    from Library import Library
    from Song import get_complete_song
    if not can_add_song():
        print 'Required tools : %s' % string.join(get_song_adding_tools())
    elif len(sys.argv) != 5:
        print 'Usage : %s %s file artist title album' % (sys.executable, sys.argv[0])
    else:
        _manager = Manager()
        _lib = Library()
        _file_path = sys.argv[1]
        _song = get_complete_song(sys.argv[2], sys.argv[3], sys.argv[4])
        _task = SongAdding(_manager, _lib, _file_path, _song)
        print 'Adding the file :'
        print '\t%s' % _task.get_src_file_path()
        print 'to the file :'
        print '\t%s' % _task.get_dest_file_path()
        print 'with song :'
        print '\t%s' % _task.get_song()
        print 'Please wait...'
        _task.start()
        print _lib

