# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from util.Directory import Directory
from util.File import File

from Task import Task
from TagsWriting import TAGS_WRITING_TOOL
from TagsWriting import TAGS_WRITING_TOOL_EXISTS
from TagsWriting import TagsWriting
from RootRemoving import RootRemoving
from SongFile import SongFile
from TagsSet import TagsSet


def get_song_editing_tools():
    return [TAGS_WRITING_TOOL]


def can_edit_song():
    return TAGS_WRITING_TOOL_EXISTS


class SongEditing(Task):
    """A task which edits a song file from a file path and a song."""

    __manager = None

    __library = None

    __song = None

    __root_removing = None

    __tags_writing = None

    def __init__(self, p_manager, p_library, p_old_file_path, p_song):
        Task.__init__(self)        
        _set = TagsSet(p_song)
        # get a temporary file path
        _dir = Directory(p_manager.get_tmp_path())
        _inode = _dir.get_inode('song.ogg')
        # create a root removing task
        self.__root_removing = RootRemoving(p_old_file_path, _inode.get_path())
        # create a tags writing task
        _song_file = p_manager.get_path_config().get_correct_file(p_song)
        self.__tags_writing = TagsWriting(_inode.get_path(), _song_file.get_path(), _set)
        # set the attributes
        self.__manager = p_manager
        self.__library = p_library
        self.__song = p_song

    def get_old_file_path(self):
        return self.__root_removing.get_src_file_path()

    def get_new_file_path(self):
        return self.__tags_writing.get_dest_file_path()

    def get_song(self):
        return self.__song.clone()

    def start(self):
        """Edit the given file name with the given song and
        return the new file path.

        Raise ToolNotFoundException if the tool for writing tags is not available.
        Raise FileNotFoundException if the file does not exist.
        Raise FileNotReadableException if the file is not readable.
        Raise FileNotOggVorbisFileException if the file is not an Ogg Vorbis file.
        Raise DirectoryNotWritableException if a directory is not writable.
        Raise ProcessException if a process stops irregularly."""
        # remove from the root directory
        self.__root_removing.start()
        # copy the file and write tags in the same time
        self.__tags_writing.start()
        # remove the song from the library
        if self.__library.has_song(self.get_old_file_path()):
            self.__library.remove_song(self.get_old_file_path())
        # add the song in the library
        self.__library.add_song(self.get_new_file_path(), self.__song)
        # update music files of all play lists
        _song_file = SongFile(self.get_new_file_path(), self.__song)
        _play_lists = self.__manager.get_play_lists_set() + [self.__manager.get_play_thread().get_queue()]
        for _pl in _play_lists:
            try:
                _pl.update(self.get_old_file_path(), _song_file)
            except IndexError:
                pass
        # remove the temporary file
        _file = File(self.__tags_writing.get_src_file_path())
        _file.remove()


if __name__ == '__main__':
    import sys
    import string
    from Manager import Manager
    from Library import Library
    from Song import get_complete_song
    if not can_edit_song():
        print 'Required tools : %s' % string.join(get_song_editing_tools())
    elif len(sys.argv) != 5:
        print 'Usage : %s %s file artist title album' % (sys.executable, sys.argv[0])
    else:
        _manager = Manager()
        _lib = Library()
        _file_path = sys.argv[1]
        _song = get_complete_song(sys.argv[2], sys.argv[3], sys.argv[4])
        _task = SongEditing(_manager, _lib, _file_path, _song)
        print 'Editing the file :'
        print '\t%s' % _task.get_old_file_path()
        print 'to the file :'
        print '\t%s' % _task.get_new_file_path()
        print 'with song :'
        print '\t%s' % _task.get_song()
        print 'Please wait...'
        _task.start()
        print _lib

