# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from util.File import FileNotFoundException
from util.File import FileNotReadableException
from util.File import File

from MusicFile import FileNotOggVorbisException
from Process import is_in_path
from Process import ToolNotFoundException
from Process import Process
from Task import Task

PLAYING_TOOL = 'ogg123'

PLAYING_TOOL_EXISTS = is_in_path(PLAYING_TOOL)


class SongPlaying(Task):
    """A task which plays an Ogg Vorbis file."""

    __file_path = ''

    __process = None

    def __init__(self, p_file_path):
        Task.__init__(self)        
        self.__file_path = p_file_path
        self.__process = None

    def get_file_path(self):
        return self.__file_path

    def start(self):
        """Play an Ogg Vorbis file.

        Raise ToolNotFoundException if the tool is not available.
        Raise FileNotFoundException if the file does not exist.
        Raise FileNotReadableException if the file is not readable.
        Raise FileNotOggVorbisFileException if the file is not an Ogg Vorbis file."""
        # verify that the rip tool is available
        if not PLAYING_TOOL_EXISTS:
            raise ToolNotFoundException, PLAYING_TOOL
        _file = File(self.get_file_path())
        # verify that the file exists
        if not _file.exists():
            raise FileNotFoundException, _file.get_path()
        # verify that the file is readable
        if not _file.is_readable():
            raise FileNotReadableException, _file.get_path()
        # verify that the file is an Ogg Vorbis file
        if not _file.has_ext('.ogg'):
            raise FileNotOggVorbisException, _file.get_path()
        # launch process
        _cmd = PLAYING_TOOL + " -q \"" + self.get_file_path() + "\"" 
        self.__process = Process(_cmd, Process.PLAY_LOG_FILE)
        self.__process.wait()



    def stop(self):
        if self.__process is not None:
            self.__process.kill()


if __name__ == '__main__':
    import sys
    if not PLAYING_TOOL_EXISTS:
        print 'Required tool : %s' % PLAYING_TOOL
    elif len(sys.argv) != 2:
        print 'Usage : %s %s file' % (sys.executable, sys.argv[0])
    else:
        _task = SongPlaying(sys.argv[1])
        print 'Playing the file :'
        print '\t%s' % _task.get_file_path()
        print 'Please wait...'
        _task.start()

