# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from util.Directory import Directory

from Task import Task
from RootRemoving import RootRemoving


class SongRemoving(Task):
    """A task which removes a song file from the file system, the library
    and all play lists."""

    __manager = None

    __library = None

    __root_removing = None

    def __init__(self, p_manager, p_library, p_file_path):
        Task.__init__(self)        
        # create a root removing task
        self.__root_removing = RootRemoving(p_file_path)
        # set the attributes
        self.__manager = p_manager
        self.__library = p_library

    def get_file_path(self):
        return self.__root_removing.get_src_file_path()

    def start(self):
        """Remove the file from the root directory and from the library and
        return the temporary file path.

        Raise FileNotFoundException if the file does not exist.
        Raise FileNotReadableException if the file cannot be read.
        Raise DirectoryNotWritableException if the file or a directory cannot
        be removable or if the temporary file cannot be created.
        Raise ProcessException if a process stops irregularly."""
        self.__root_removing.start()
        # remove the song from the library
        if self.__library.has_song(self.get_file_path()):
            self.__library.remove_song(self.get_file_path())
        # remove files with path in all play lists
        _play_lists = self.__manager.get_play_lists_set() + [self.__manager.get_play_thread().get_queue()]
        for _pl in _play_lists:
            try:
                _pl.remove_by_path(self.get_file_path())
            except IndexError:
                pass
        


if __name__ == '__main__':
    import sys
    from Manager import Manager
    from Library import Library
    if len(sys.argv) != 2:
        print 'Usage : %s %s file' % (sys.executable, sys.argv[0])
    else:
        _manager = Manager()
        _lib = Library()
        _file_path = sys.argv[1]
        _task = SongRemoving(_manager, _lib, _file_path)
        print 'Removing the file :'
        print '\t%s' % _task.get_file_path()
        print 'Please wait...'
        _task.start()
        print _lib

