# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from util.String import String
from util.File import FileNotFoundException
from util.File import FileNotReadableException
from util.File import File

from MusicFile import FileNotOggVorbisException
from TagsSet import TagsSet
from Task import Task
from Process import is_in_path
from Process import ToolNotFoundException
from Process import Process

TAGS_READING_TOOL = "vorbiscomment"

TAGS_READING_TOOL_EXISTS = is_in_path(TAGS_READING_TOOL)


class TagsReading(Task):
    """A task which reads tags of an ogg Vorbis file."""

    __file_path = ""

#    __song = ""

    __process = None

    def __init__(self, p_file_path):
        Task.__init__(self)        
        self.__file_path = p_file_path
#        self.__song = None
        self.__process = None

    def get_file_path(self):
        return self.__file_path

#    def get_song(self):
#        return self.__song


    def start(self):
        """Read tags .

        Raise ToolNotFoundException if the tool for reading tags is not available.
        Raise FileNotFoundException if the file does not exist.
        Raise FileNotReadableException if the file is not readable.
        Raise FileNotOggVorbisException if the file has not the Ogg Vorbis extension."""
        _set = TagsSet()
        # verify that the tool exists
        if not TAGS_READING_TOOL_EXISTS:
            raise ToolNotFoundException, TAGS_READING_TOOL
        _file = File(self.get_file_path())
        # verify that the file exists
        if not _file.exists():
            raise FileNotFoundException, _file.get_path()
        # verify that the file can be read
        if not _file.is_readable():
            raise FileNotReadableException, _file.get_path()
        # verify that the file has an Ogg Vorbis extension
        if not _file.has_ext('.ogg'):
            raise FileNotOggVorbisException, _file.get_path()
        # launch process
        _cmd = TAGS_READING_TOOL + " -l \"" + self.get_file_path() + "\""
        self.__process = Process(_cmd, Process.IO_LOG_FILE)
        # catchs all comments
        _res = self.__process.readline()
        while _res != "":
            _groups = _res.split('=', 1)
            _key = _groups[0].lower()
            _data = _groups[1].strip()
            _set.add_tag(_key, _data)
            _res = self.__process.readline()
        # waits for the end of the process
        self.__process.wait()
        # converts the tags set in a song
        _song = _set.to_song()
        # sets the artist to the disc, if the artist name is in the path
        _artist = _song.get_artist().get_main_artist()
        _expr = str(_artist)
        if _expr:
            _expr = _expr.replace(" ", ".*")
            _file = File(self.get_file_path())
            _dir = _file.get_parent_directory()
            _dir_str = String(str(_dir))
    #        _path = os.path.dirname(p_file_path)
            if _dir_str.matchs(_expr):
                _song.get_disc().set_artist(_artist)
        # set the song
#        self.__song = _song
        # return the song
        return _song

    
    def stop(self):
        if self.__process is not None:
            self.__process.kill()


if __name__ == '__main__':
    import sys
    if not TAGS_READING_TOOL_EXISTS:
        print 'Required tool : %s' % TAGS_READING_TOOL
    elif len(sys.argv) != 2:
        print 'Usage : %s %s file' % (sys.executable, sys.argv[0])
    else:
        _task = TagsReading(sys.argv[1])
        print 'Reading the file :'
        print '\t%s' % _task.get_file_path()
        print 'Please wait...'
        _task.start()
        print 'Song :'
        print '\t%s' % _task.get_song()

