# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#import threading

from util.QueuedThread import QueuedThread
from util.UserException import UserException

from Process import ProcessKilledException


class TasksThread(QueuedThread):
    """A thread that executes a tasks queue."""

    __callback = None

    __errors = []

    def __init__(self):
        QueuedThread.__init__(self, "Tasks thread")
        self.__callback = None
        self.__errors = []

    ###########################################################################
    # ACCESSORS
    ###########################################################################

    def get_queue(self):
        """Return a copy of the tasks queue, including the current task."""
        _ret = []
        
        _task = self.get_current_task()
        if _task:
            _ret.append(_task)
        _tasks = QueuedThread.get_queue(self)
        for _task in _tasks:
            _ret.append(_task)
        return _ret

    def get_current_task(self):
        """Return the task which is currently handled."""
        return self.get_current_item()

    def get_errors(self):
        """Return the errors list.

        Return a copy of the errors list so that users cannot modify it."""
        _ret = []
        for _e in self.__errors:
            _ret.append(_e)
        return _ret


    #################################################
    # QUEUE MANAGEMENT
    #################################################

    def add_task(self, p_task):
        QueuedThread.get_queue(self).append(p_task)
#        self.__changed()
        self.restart()

    def cancel_task(self, p_index):
        """Cancel the task of the given index.

        Raise IndexError if the index is out of bound."""
        if p_index == 0:
            self.get_current_task().stop()
#            _task = self.get_queue().get_first()
#            _task.stop()
        else:
            QueuedThread.get_queue(self).delete(p_index - 1)
#        self.__changed()

    #################################################
    # TASK MANAGEMENT
    #################################################

#    def __changed(self):
#        """Call the function."""
#        if self.__callback is not None:
#            apply(self.__callback)
        
    def __notify_error(self, p_exception):
        """Add the given exception to the errors list."""
        self.__errors.append(p_exception)

    def stop_item_handling(self):
        """Stop the current task."""
        _task = self.get_current_task()
        if _task is not None:
            _task.stop()

    def handle_item(self, p_task):
        """Handle the given task."""
        try:
            p_task.start()
        except ProcessKilledException, _exc:
            pass
        except UserException, _exc:
            self.__notify_error(_exc)
