# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from util.String import String
from util.NumberString import NumberString

TUNE_TAG = "tune"

TITLE_TAG = "title"

YEAR_TAG = "year"


class Tune:
    """A tune.

    Warning ! For a reason of performance, this class is not synchronized.
    Don't apply its modificators to a shared instance.
    """

    __title = String("Title")

    __sec_title = String("Second Title")

    __year = NumberString("")

    def __init__(self, p_title, p_year = ""):
        """Construct a tune."""
        self.__title = String(p_title)
        self.__sec_title = String("")
        self.__year = NumberString(p_year)

    def get_title(self):
        """Return the title."""
        return self.__title.data

    def set_title(self, p_title):
        """Set the title."""
        self.__title = String(p_title)

    def get_sec_title(self):
        """Return the second title."""
        return self.__sec_title.data

    def set_sec_title(self, p_sec_title):
        """Set the second title."""
        self.__sec_title = String(p_sec_title)

    def get_year(self):
        """Return the year."""
        return self.__year.data

    def set_year(self, p_year):
        """Set the year."""
        self.__year = NumberString(p_year)

    def get_entire_title(self):
        """Return the main title and the second title in parenthesis."""
        _ret = self.get_title()
        _sec = self.get_sec_title()
        if _sec != "":
            _ret += " (" + _sec + ")"
        return _ret

    def matchs_title(self, p_expr):
        """Determine if the tune has a title."""
        _str = String(self.get_entire_title())
        return _str.matchs(p_expr)
        
    def clone(self):
        """Return a clone."""
        _tune = Tune(self.get_title(), self.get_year())
        return _tune

    def matchs(self, p_tune):
        """Match with another tune."""
        _ret = 1
        if p_tune is not None:
            _ret = self.matchs_title(p_tune.get_title())
            if _ret:
                _ret = self.__year.matchs(p_tune.get_year())
        return _ret

    def __cmp__(self, p_tune):
        """Compare with another tune."""
        _ret = 0
        if p_tune is not None:
            _ret = cmp(self.__title, p_tune.__title)
            if _ret == 0:
                _ret = cmp(self.__year, p_tune.__year)
        return _ret

    def __str__(self):
        """Return a representation as a string."""
        return self.get_entire_title() + " (" + self.get_year() + ")"

    def write_xml(self, p_writer):
        """Write a represetation as a XML string."""
        _writer = p_writer
        _writer.write_begin_tag(TUNE_TAG)
        _writer.write_tags_with_data(TITLE_TAG, self.get_title())
        _writer.write_tags_with_data(YEAR_TAG, self.get_year())
        _writer.write_end_tag(TUNE_TAG)
        return _writer
