# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from base.Manager import Manager
from base.PlayList import PlayList
from base.SongAdding import get_song_adding_tools
from base.SongAdding import can_add_song
from base.MP3Importing import get_mp3_importing_tools
from base.MP3Importing import can_import_mp3
from base.TrackImporting import get_track_importing_tools
from base.TrackImporting import can_import_track
from base.LibraryRefreshing import get_library_refreshing_tools
from base.LibraryRefreshing import can_refresh_library

from Page import Page
from LibraryList import LibraryList
from PlayListsSetMenu import PlayListsSetMenu
from PlayListPage import PlayListPage
from OggVorbisAddingPage import OggVorbisAddingPage
from MP3ImportingPage import MP3ImportingPage
from TrackImportingPage import TrackImportingPage
from LibraryRefreshingPage import LibraryRefreshingPage
from MissingToolsPage import MissingToolsPage
from AboutPage import AboutPage
from Menu import Menu
from Session import Session

class LibraryPage(Page):
    """A page of the library"""

    __list = None

    def __init__(self):
        Page.__init__(self, 0)
        self.__list = LibraryList()
        self.__list.show()
        self.add(self.__list)

    def update(self):
        """Update the page."""
        self.__list.update()
        Page.update(self)

    def get_title(self):
        """Return the title of the page."""
        return 'Library'

    def get_menu(self):
        """Return the popup menu."""
        _menu = Menu()
        _sub_menu1 = Menu()
        _sub_menu1.append_item('Ogg Vorbis file...', self.__import_ov)
        _sub_menu1.append_item('MP3 file...', self.__import_mp3)
        _sub_menu1.append_item('CDDA track...', self.__import_track)
        _menu.append_menu('Import', _sub_menu1)
        _menu.append_item('Refresh...', self.__refresh)
        _sub_menu = PlayListsSetMenu(self.__open)
        _menu.append_menu('Open play list', _sub_menu)
        _menu.append_item('Create a new play list', self.__create)
        _menu.append_separator()
        _menu = self.__list.append_menu(_menu)
        _menu.append_separator()
        _menu.append_item('About...', self.__about)
        return _menu

    def get_status(self):
        """Return the status message."""
        return self.__list.get_status()

    ###########################################################################
    # CALLBACKS
    ###########################################################################

    def __import_ov(self, p_menu_item):
        """Open a page to add an Ogg Vorbis file into the library."""
#        print '__import_ov', self, p_menu_item
        _session = Session()
        if not can_add_song():
            _msg = 'Sorry but you cannot import an Ogg Vorbis file.'
            _tools = get_song_adding_tools()
            _mtpage = MissingToolsPage(_msg, _tools)
            # display the page
            _session.get_main_window().add_tab(_mtpage)
        else:
            _page = OggVorbisAddingPage()
            # display the page
            _session.get_main_window().add_tab(_page)

    def __import_mp3(self, p_menu_item):
        """Open a page to add a MP3 file into the library."""
#        print '__import_mp3', self, p_menu_item
        _session = Session()
        if not can_import_mp3():
            _msg = 'Sorry but you cannot import a MP3 file.'
            _tools = get_mp3_importing_tools()
            _mtpage = MissingToolsPage(_msg, _tools)
            # display the page
            _session.get_main_window().add_tab(_mtpage)
        else:
            _page = MP3ImportingPage()
            # display the page
            _session.get_main_window().add_tab(_page)

    def __import_track(self, p_menu_item):
        """Open a page to add a CDDA track into the library."""
#        print '__import_track', self, p_menu_item
        _session = Session()
        if not can_import_track():
            _msg = 'Sorry but you cannot import a CDDA track.'
            _tools = get_track_importing_tools()
            _mtpage = MissingToolsPage(_msg, _tools)
            # display the page
            _session.get_main_window().add_tab(_mtpage)
        else:
            _page = TrackImportingPage()
            # display the page
            _session.get_main_window().add_tab(_page)

    def __refresh(self, p_menu_item):
        """Open a page to refresh the library."""
#        print '__refresh', self, p_menu_item
        _session = Session()
        if not can_refresh_library():
            _msg = 'Sorry but you cannot refresh the library.'
            _tools = get_library_refreshing_tools()
            _mtpage = MissingToolsPage(_msg, _tools)
            # display the page
            _session.get_main_window().add_tab(_mtpage)
        else:
            _page = LibraryRefreshingPage()
            # display the page
            _session.get_main_window().add_tab(_page)
        
    def __open(self, p_play_list):
        """Open a page to edit the given play list."""
#        print '__open', self, p_play_list
        # create a page and display it
        _page = PlayListPage(p_play_list)
        # display the page
        _session = Session()
        _session.get_main_window().add_tab(_page)
        
    def __create(self, p_menu_item):
        """Create a new empty play list and open a page to edit it."""
#        print '__create', self, p_menu_item
        # create a new play list
        _pl = PlayList('Play list')
        # add the play list to the manager
        _manager = Manager()
        _manager.get_play_lists_set().append(_pl)
        # create a page and display it
        _page = PlayListPage(_pl)
        # display the page and set play lists as modified
        _session = Session()
        _session.get_main_window().add_tab(_page)
        _session.set_play_lists_as_modified()

    def __about(self, p_menu_item):
        """Open a page to display a message about the project."""
#        print '__about', self, p_menu_item
        # create a page and display it
        _page = AboutPage()
        # display the page
        _session = Session()
        _session.get_main_window().add_tab(_page)
        
