# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk

from base.Manager import Manager

from Page import Page
from PathFrame import PathFrame
from SimplePageButtonBox import SimplePageButtonBox
from Session import Session


class LibraryRefreshingPage(Page):
    """A page for asking the root directory and for refreshing the library."""

    __path_frame = None

    def __init__(self):
        Page.__init__(self)
        # creation of a first line
        self.__path_frame = PathFrame('Root directory', 1)
        self.__path_frame.show()
        self.pack(self.__path_frame, gtk.FALSE)
        # creation of a second line
        _box = SimplePageButtonBox(self.__apply, self.__cancel)
        _box.show()
        self.pack(_box, gtk.FALSE)
        # update the frame
        self.update()
        
    def update(self):
        """Update the page."""
        Page.update(self)
        _manager = Manager()
        _path = _manager.get_root_path()
        self.__path_frame.set_path(_path)

    def get_title(self):
        """Return the title of the page."""
        return 'Library refreshing'


    ###########################################################################
    # CALLBACKS
    ###########################################################################

    def __apply(self):
        """Add the library refreshing task and close the page."""
#        print '__apply', self
        _manager = Manager()
        _path = self.__path_frame.get_path()
        _task = _manager.get_library_refreshing(_path)
        _manager.get_tasks_thread().add_task(_task)
        # display the page
        _session = Session()
        _session.set_library_as_modified()
        _session.get_main_window().close_current_page()
    
    def __cancel(self):
        """Close the page."""
#        print '__cancel', self
        _session = Session()
        _session.get_main_window().close_current_page()

    
    
