# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk

from Session import Session
from Menu import Menu

class List(gtk.GtkScrolledWindow):
    """A multi-columned list."""

    __clist = None

    __sequence = []

    __selected_items = []

    __rows_number = 0

    def __init__(self, p_sequence, p_col_num, p_titles = None):
        gtk.GtkScrolledWindow.__init__(self)
        self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        # initialize the attributes
        self.__sequence = p_sequence
        self.__selected_items = {}
        self.__rows_number = 0
        # create the multi-column list
        self.__clist = gtk.GtkCList(p_col_num, p_titles)
        self.__clist.column_titles_passive()
        self.__clist.set_selection_mode(gtk.SELECTION_MULTIPLE)
        self.__clist.connect('select_row', self.__row_selected)
        self.__clist.connect('unselect_row', self.__row_unselected)
        self.__clist.connect('button_press_event', self.__row_clicked)
        self.__clist.show()
        # add the list into the scrolled window
        self.add(self.__clist)

    def get_sequence(self):
        """Return the sequence displayed bu the list."""
        return self.__sequence

    def set_sequence(self, p_sequence):
        """Set the sequence that the list must display.

        Warning : you must use the update function to display the new
        sequence."""
        self.__sequence = p_sequence

    def get_selected_rows(self):
        """Return a sorted list of the selected rows."""
        _ret = self.__selected_items.keys()
        _ret.sort()
        return _ret

    def get_selected_items(self):
        """Return the selected items sequence sorted by their rows."""
        _ret = []
        _rows = self.get_selected_rows()
        for _row in _rows:
            _ret.append(self.__selected_items[_row])
        return _ret

    def __append(self, p_tuple):
        """Add a row to list."""
        self.__rows_number = self.__clist.append(p_tuple)
        if self.__rows_number % 2:
            self.__clist.set_background(self.__rows_number, gtk.GdkColor(-0x4000, -0x4000, -0x1))

    def set_widths(self, p_widths):
        """Set the width of all columns from the given width sequence."""
        _num = 0
        for _width in p_widths:
            self.__clist.set_column_width(_num, _width)
            _num += 1

    def select_row(self, p_index):
        """Selected the row given by the index."""
        self.__clist.select_row(p_index, 0)

    def invert(self):
        """Invert the selection."""
        # make a copy of the selected rows index
        _rows = self.get_selected_rows()
        # unselect the selected rows
        for _row in _rows:
            self.__clist.unselect_row(_row, 0)
        # select the other rows
        for _row in range(self.__rows_number + 1):
            if _row not in _rows:
                self.__clist.select_row(_row, 0)

    def select_all(self):
        """Select all rows."""
        self.__clist.select_all()

    def unselect_all(self):
        """Unselect all rows."""
        self.__clist.unselect_all()

    def get_tuple(self, p_item):
        """Return the tuple corresponding to the given item.

        Derived class must redefine the method."""
        return [str(p_item)]


    def __search_item(self, p_item, p_begin_row):
        """Search the given item in the end of the sequence
        beginning by the given row."""
        _ret = -1
#        print 'in'
#        for _item in self.__sequence[p_begin_row:]:
#            print '+', _item
        try:
            _ret = self.__sequence[p_begin_row:].index(p_item) + p_begin_row
        except ValueError:
            pass
        return _ret

    """
    def update_selection(self):
        # get all selected items before unselect them
        _items = self.get_selected_items()
        self.unselect_all()
        # search new rows corresponding the old selection
        _rows = []
        for _item in _items:
            _row = self.__get_row(_item, 0)
            while _row != -1 and _row in _rows:
                _row = self.__get_row(_item, _row)
            if _row != -1:
                _rows.append(_row)
        # select new rows
        for _row in _rows:
            self.select_row(_row)
    """

    def __get_new_selected_rows(self):
        """Return the list of rows which must be selected considering
        the sequence."""
#        print '__get_new_selected_rows'
        _ret = []
        # browse all selected (row, item)
        _doubles = self.__selected_items.items()
        for (_row, _item) in _doubles:
            if _row < len(self.__sequence) \
            and self.__sequence[_row] == _item \
            and _row not in _ret:
                # if the item is in the same place in the sequence
                # or if the row is not in the new rows list
                # we append the row to the list
#                print 'item did not move', _item
                _ret.append(_row)
            else:
                # we search a row with the same item
#                print 'search item', _item
#                print 'with ', _ret
                _row = self.__search_item(_item, 0)
#                print 'result', _row
                while _row != -1 and _row < len(self.__sequence) and _row in _ret:
#                    print 'research item'
                    _row = self.__search_item(_item, _row + 1)
#                    print 'result', _row
                if _row != -1:
#                    print 'OK'
                    _ret.append(_row)
#                print 'new _ret', _ret
        return _ret

    def update(self):
        """Update the list from the given sequance."""
#        _rows = self.get_selected_rows()
        _rows = self.__get_new_selected_rows()
        # unselect all rows
        self.unselect_all()
        # get the vertical adjustment and its current value
        _adj = self.get_vadjustment()
        _value = _adj.value
        # freeze and clear the list
        self.__clist.freeze()
        self.__clist.clear()
        # insert a representation of all items into the list
        for _item in self.__sequence:
            self.__append(self.get_tuple(_item))
        # reselect rows
        for _row in _rows:
            
            self.select_row(_row)
        # change the current value of the vertical adjustement
        _adj.set_value(_value)
        # unfreeze the list
        self.__clist.thaw()

    def get_status(self):
        """Return the status message."""
        _ret = '%d items - %d selected items' % (len(self.get_sequence()), len(self.get_selected_items()))
        return _ret

    def get_menu(self):
        """Return the selection popup menu.

        It is a convenience way for derived class to redfine this method."""
        return None

    def append_menu(self, p_menu = None):
        """Add global menu items to the given menu

        Create a new menu if the given menu is None."""
        _menu = p_menu
        if _menu == None:
            _menu = Menu()
        _menu.append_item('Invert selection', self.__invert)
        _menu.append_item('Select all', self.__select_all)
        _menu.append_item('Unselect all', self.__unselect_all)
        return _menu


    ###########################################################################
    # CALLBACKS
    ###########################################################################

    def __row_clicked(self, p_list, p_event):
        """Popup the selection menu if the third button was clicked."""
#        print '__row_clicked !', self, p_list, p_event
        if p_event.button == 3:
            _cell = p_list.get_selection_info(p_event.x, p_event.y)
            if _cell is not None:
                _row = _cell[0]
                _col = _cell[1]
                p_list.select_row(_row, _col)
            _menu = self.get_menu()
            if _menu is not None:
                _menu.show()
                _menu.popup(None, None, None, p_event.button, p_event.time)
            self.__clist.emit_stop_by_name('button_press_event')
        return 1

    def __row_selected(self, p_list, p_row, p_type, p_event):
        """Keep the new selected row and update the status bar."""
#        print '__row_selected', self, p_list, p_row, p_type, p_event
        try:
            _item = self.__sequence[p_row]
        except IndexError:
            pass
        else:
                self.__selected_items[p_row] = _item
                _session = Session()
                _session.get_main_window().set_status(self.get_status())
            
    def __row_unselected(self, p_list, p_row, p_type, p_event):
        """Throw the new unselected row and update the status bar."""
#        print '__row_unselected', self, p_list, p_row, p_type, p_event
        if self.__selected_items.has_key(p_row):
            del self.__selected_items[p_row]
            _session = Session()
            _session.get_main_window().set_status(self.get_status())
            
    def __invert(self, p_menu_item):
        """Invert the selection."""
#        print '__invert', self, p_menu_item
        self.invert()

    def __select_all(self, p_menu_item):
        """Select all rows."""
#        print '__select_all', self, p_menu_item
        self.select_all()

    def __unselect_all(self, p_menu_item):
        """Unselect all rows."""
#        print '__unselect_all', self, p_menu_item
        self.unselect_all()
