# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk

class Menu(gtk.GtkMenu):
    """A menu."""

    def __init__(self):
        gtk.GtkMenu.__init__(self)
        
    def __create_item(self, p_title, p_call_back, p_data):
        """Return a menu item which will have the given title and call the
        given function with the given data."""
        _menu_item = gtk.GtkMenuItem(p_title)
        if p_data is not None:
            _menu_item.connect('activate', p_call_back, p_data)
        else:
            _menu_item.connect('activate', p_call_back)
        return _menu_item

    def prepend_item(self, p_title, p_call_back, p_data = None):
        """Add, at the beginning of the menu, a menu item which will have
        the given title and call the given function with the given data."""
        _menu_item = self.__create_item(p_title, p_call_back, p_data)
        self.prepend(_menu_item)
        _menu_item.show()

    def append_item(self, p_title, p_call_back, p_data = None):
        """Add, at the end of the menu, a menu item which will have
        the given title and call the given function with the given data."""
        _menu_item = self.__create_item(p_title, p_call_back, p_data)
        self.append(_menu_item)
        _menu_item.show()

    def prepend_separator(self):
        """Add, at the beginning of the menu, a separator."""
        _menu_item = gtk.GtkMenuItem()
        self.prepend(_menu_item)
        _menu_item.show()

    def append_separator(self):
        """Add, at the end of the menu, a separator."""
        _menu_item = gtk.GtkMenuItem()
        self.append(_menu_item)
        _menu_item.show()
        
    def append_menu(self, p_title, p_menu):
        """Add, at the end of the menu, a menu item which will have
        the given title and open the given submenu."""
        _menu_item = gtk.GtkMenuItem(p_title)
        _menu_item.set_submenu(p_menu)
        p_menu.show()
        self.append(_menu_item)
        _menu_item.show()

