# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk
import os

from util.File import FileNotFoundException
from util.File import FileNotReadableException
from util.Inode import Inode

from base.Manager import Manager
from base.MusicFile import FileNotOggVorbisException
from base.TagsReading import TAGS_READING_TOOL_EXISTS
from base.TagsReading import TagsReading

from Page import Page
from Menu import Menu
from PathFrame import PathFrame
from SongFrame import SongFrame
from PathConfigFrame import PathConfigFrame
from SimplePageButtonBox import SimplePageButtonBox
from Session import Session

last_file_path = os.environ['HOME']

last_song = None

class OggVorbisAddingPage(Page):
    """A page for adding an Ogg Vorbis file."""

    __path_frame = None

    __song_frame = None

    __path_config_frame = None

    def __init__(self):
        Page.__init__(self)
        # add a file name frame
        self.__path_frame = PathFrame('Ogg Vorbis file')
        self.__path_frame.show()
        self.pack(self.__path_frame, gtk.FALSE)
        if TAGS_READING_TOOL_EXISTS:
            # add a button to read tags from file
            _button = gtk.GtkButton('Update song from file')
            _button.connect('clicked', self.__update)
            _button.show()
        self.pack(_button, gtk.FALSE)
        # add a song frame
        self.__song_frame = SongFrame('Song', gtk.TRUE, self.__song_changed)
        self.__song_frame.show()
        self.pack(self.__song_frame, gtk.FALSE)
        # add a path config frame
        self.__path_config_frame = PathConfigFrame('Path configuration', self.__path_config_changed)
        self.__path_config_frame.show()
        self.pack(self.__path_config_frame, gtk.FALSE)
        # add buttons
        _button_box = SimplePageButtonBox(self.__apply, self.__cancel)
        _button_box.show()
        self.pack(_button_box, gtk.FALSE)
        # initialize the path config frame
        _manager = Manager()
        _config = _manager.get_path_config()
        self.__path_config_frame.set_path_config(_config)
        # initialize the song frame with a blank title
        if last_song is not None:
            _song = last_song.clone()
            _song.get_tune().set_title('')
            self.__song_frame.set_song(_song)
        # initialize the path frame with the previous directory path
        _inode = Inode(last_file_path)
        if not _inode.is_directory():
            _inode = _inode.get_parent()
        self.__path_frame.set_path(_inode.get_path())

    """        
    def update(self):
        Page.update(self)
    """

    def get_title(self):
        """Return the title of the page."""
        return 'Ogg Vorbis importing'


    ###########################################################################
    # CALLBACKS
    ###########################################################################

    def __update(self, p_button):
        """Read the vorbis comment from the entered file path
        and update the song frame."""
#        print '__update', self, p_button
        _path = self.__path_frame.get_path()
        _task = TagsReading(_path)
        try:
            _song = _task.start()
        except FileNotFoundException, _exc:
            _session = Session()
            _session.get_main_window().set_status('The file %s does not exist !' % _exc)
        except FileNotReadableException, _exc:
            _session = Session()
            _session.get_main_window().set_status('The file %s is not readable.' % _exc)
        except FileNotOggVorbisException, _exc:
            _session = Session()
            _session.get_main_window().set_status('The file %s is not an Ogg Vorbis file.' % _exc)
        else:
            self.__song_frame.set_song(_song)

    def __song_changed(self, p_new_song):
        """Update the example of the path configuration frame
        with the new song."""
#        print '__song_changed', self, p_new_song
        if self.__path_config_frame:
            self.__path_config_frame.update_example(p_new_song)

    def __path_config_changed(self, p_new_config):
        """Update the example of the path configuration frame
        with the new configuration."""
#        print '__path_config_changed', self, p_new_config
        _song = self.__song_frame.get_song()
        self.__path_config_frame.update_example(_song)

    def __cancel(self):
        """Close the page."""
#        print '__cancel', self
        # close the page
        _session = Session()
        _session.get_main_window().close_current_page()

    def __apply(self):
        """Add the song adding task, set the library as modified
        and close the page."""
        global last_file_path, last_song
#        print "__apply", self
        _manager = Manager()
        # set the path config
        _config = self.__path_config_frame.get_path_config()
        _manager.set_path_config(_config)
        # get path and song
        last_file_path = self.__path_frame.get_path()
        last_song = self.__song_frame.get_song()
        # get the task and add it to the tasks thread
        _task = _manager.get_song_adding(last_file_path, last_song)
        _manager.get_tasks_thread().add_task(_task)
        # set the library as modified and close the page
        _session = Session()
        _session.set_library_as_modified()
        _session.get_main_window().close_current_page()
