# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk

from base.PathConfig import PathConfig

from Widgets import Entry
from Widgets import CheckButton

class PathConfigFrame(gtk.GtkFrame):
    """A frame for editing a path configuration."""

    __format_entry = None

    __disc_format_entry = None

    __artists_sep_entry = None

    __space_entry = None

    __lower_checkbutton = None

    __example_label = None

    __changed_callback = None

    def __init__(self, p_title, p_changed_callback):
        gtk.GtkFrame.__init__(self, p_title)
        self.__changed_callback = p_changed_callback
        _table = gtk.GtkTable(4, 4, gtk.FALSE)
        # add the format label
        _label = gtk.GtkLabel('Song format')
        _label.show()
        _table.attach(_label, 0, 1, 0, 1)
        # add the format entry
        self.__format_entry = Entry(gtk.TRUE,
                                     'File path format. Use %t for title, %a for artist(s), %d for disc, %g for genre, %r for track, %y for year',
                                     self.__changed)
        self.__format_entry.show()
        _table.attach(self.__format_entry, 1, 2, 0, 1)
        # add the disc format label
        _label = gtk.GtkLabel('Disc format')
        _label.show()
        _table.attach(_label, 2, 3, 0, 1)
        # add the disc format entry
        self.__disc_format_entry = Entry(gtk.TRUE,
                                          'Disc format used when artist is specified. Use %t for disc title, %a for disc artist',
                                          self.__changed)
        self.__disc_format_entry.show()
        _table.attach(self.__disc_format_entry, 3, 4, 0, 1)
        # add the artist separator label
        _label = gtk.GtkLabel('Artist separator')
        _label.show()
        _table.attach(_label, 0, 1, 1, 2)
        # add the artist separator entry
        self.__artists_sep_entry = Entry(gtk.TRUE, 'Field that must separate artist names', self.__changed)
        self.__artists_sep_entry.show()
        _table.attach(self.__artists_sep_entry, 1, 2, 1, 2)
        # add the space substitute label
        _label = gtk.GtkLabel('Space substitute')
        _label.show()
        _table.attach(_label, 2, 3, 1, 2)
        # add the space substitute entry
        self.__space_entry = Entry(gtk.TRUE, 'String that must replace spaces', self.__changed)
        self.__space_entry.show()
        _table.attach(self.__space_entry, 3, 4, 1, 2)
        """
        _radio1 = gtk.GtkRadioButton(None, 'Replace spaces by underscores')
        _radio1.show()
        _box.pack_start(_radio1, gtk.FALSE)
        _radio2 = gtk.GtkRadioButton(_radio1, 'Delete spaces')
        _radio2.show()
        _box.pack_start(_radio2, gtk.FALSE)
        _radio3 = gtk.GtkRadioButton(_radio2, 'Do not replace spaces')
        _radio3.show()
        _box.pack_start(_radio3, gtk.FALSE)
        """
        # add the lower case check button
        self.__lower_checkbutton = CheckButton('Lowercase path', 'Must the file path be lowercased ?', self.__changed)
        """
        self.__lower_checkbutton = gtk.GtkCheckButton('Lowercase path')
        self.__lower_checkbutton.connect('toggled', self.__changed)
        """
        self.__lower_checkbutton.show()
        _table.attach(self.__lower_checkbutton, 0, 2, 2, 3)
        # add the example label
        self.__example_label = gtk.GtkLabel('')
#        self.update_example(None)
        self.__example_label.show()
        _table.attach(self.__example_label, 0, 4, 3, 4)
        # show the whole box and add it to the frame
        _table.show()
        self.add(_table)


    def set_path_config(self, p_path_config):
        """Display the given path config."""
        self.__format_entry.set_text(p_path_config.get_format())
        self.__disc_format_entry.set_text(p_path_config.get_disc_format())
        self.__artists_sep_entry.set_text(p_path_config.get_artists_sep())
        self.__space_entry.set_text(p_path_config.get_space_substitute())
        self.__lower_checkbutton.set_active(p_path_config.is_lowercasing())

    def update_example(self, p_song):
        """Update the example label with the given song
        and with the current path configuration."""
        _path = ''
        if p_song is not None:
            _config = self.get_path_config()
            _path = _config.get_correct_file_path(p_song)
        _text = 'File path : %s' % _path
        self.__example_label.set_text(_text)


    def get_path_config(self):
        """Return the current entered path configuration."""
        _config = PathConfig()
        _format = self.__format_entry.get_text()
        _config.set_format(_format)
        _disc_format = self.__disc_format_entry.get_text()
        _config.set_disc_format(_disc_format)
        _sep = self.__artists_sep_entry.get_text()
        _config.set_artists_sep(_sep)
        _substitute = self.__space_entry.get_text()
        _config.set_space_substitute(_substitute)
        _lower = self.__lower_checkbutton.get_active()
        _config.set_lowercasing(_lower)
        return _config

    ###########################################################################
    # CALLBACKS
    ###########################################################################

    def __changed(self, p_entry):
        """Call the callback function."""
#        print '__changed', self, p_entry
        if self.__changed_callback:
            _config = self.get_path_config()
            self.__changed_callback(_config)
    
