# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk

from util.Inode import Inode
from Widgets import Combo

class PathFrame(gtk.GtkFrame):
    """A frame for editing a path."""

    __path_combo = None

    __only_dir = 0

    __signal_id = 0

    __paths = []

    def __init__(self, p_title, p_only_dir = 0):
        gtk.GtkFrame.__init__(self, p_title)
        _box = gtk.GtkHBox()
        self.__only_dir = p_only_dir
        # add a label
        _label = gtk.GtkLabel('Path')
        _label.set_justify(gtk.JUSTIFY_LEFT)
        _label.show()
        _box.pack_start(_label, gtk.FALSE, gtk.FALSE, 20)
        # add an entry
        self.__path_combo = Combo(gtk.TRUE, 'Absolute path')
        self.__signal_id = self.__path_combo.entry.connect('changed', self.__entry_changed)
        self.__path_combo.show()
        _box.pack_start(self.__path_combo)
        # add the box
        _box.show()
        self.add(_box)


    def set_path(self, p_path):
        """Display the given path."""
        self.__path_combo.entry.signal_handler_block(self.__signal_id)
        self.__path_combo.entry.set_text(p_path) 
        self.__path_combo.entry.signal_handler_unblock(self.__signal_id)

    def get_path(self):
        """Return the current entered path."""
        return self.__path_combo.entry.get_text().strip()

        
    ###########################################################################
    # CALLBACKS
    ###########################################################################

    def __entry_changed(self, p_entry):
        """Try to complete the entered path."""
#        print '__entry_changed', self, p_entry
        # search a corresponding path
        _path = self.get_path()
#        print 'Path entered :', _path
        if _path:
            _inode = Inode(_path)
            _paths = _inode.complete(self.__only_dir)
#            print 'Paths choice :', _paths
            if _paths:
                self.__paths = _paths
            else:
                self.__paths = [_path]
            gtk.idle_add(self.__set_path)
                

    def __set_path(self):
        """Set the new text and select region form the current position
        to the end."""
#        print '__set_path', self
        _pos = self.__path_combo.entry.get_position()
        # set the new path
        self.__path_combo.entry.signal_handler_block(self.__signal_id)
        self.__path_combo.set_popdown_strings(self.__paths)
        self.__path_combo.entry.signal_handler_unblock(self.__signal_id)
        # select the non user-entered text
        self.__path_combo.entry.set_position(_pos)
        self.__path_combo.entry.select_region(_pos, -1)
        
