# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from SelectionList import SelectionList
from Session import Session

class PlayListList(SelectionList):
    """A list that displays a play list."""

    def __init__(self, p_play_list):
        SelectionList.__init__(self, p_play_list, 1)
        
    ###########################################################################
    # ACCESSORS
    ###########################################################################

    def get_play_list(self):
        """Return the play list."""
        return self.get_selection()

    ###########################################################################
    # MODIFICATORS
    ###########################################################################

    def set_play_list(self, p_play_list):
        """Set the given play list.

        Warning : the update method must be called to display
        the new play list."""
        self.set_selection(p_play_list)

    def sort_by_title(self):
        """Sort the list by title."""
        SelectionList.sort_by_title(self)
        _session = Session()
        _session.set_play_lists_as_modified()
        
    def sort_by_year(self):
        """Sort the list by year."""
        SelectionList.sort_by_year(self)
        _session = Session()
        _session.set_play_lists_as_modified()
        
    def sort_by_artist(self):
        """Sort the list by artist."""
        SelectionList.sort_by_artist(self)
        _session = Session()
        _session.set_play_lists_as_modified()
        
    def sort_by_disc(self):
        """Sort the list by disc."""
        SelectionList.sort_by_disc(self)
        _session = Session()
        _session.set_play_lists_as_modified()
        
    def sort_by_track(self):
        """Sort the list by track."""
        SelectionList.sort_by_track(self)
        _session = Session()
        _session.set_play_lists_as_modified()
        
    def sort_by_genre(self):
        """Sort the list by genre."""
        SelectionList.sort_by_genre(self)
        _session = Session()
        _session.set_play_lists_as_modified()
        
    def randomize(self):
        """Randomize the play list."""
        SelectionList.randomize(self)
        _session = Session()
        _session.set_play_lists_as_modified()

    def clear(self):
        """Clear the list."""
        SelectionList.clear(self)
        _session = Session()
        _session.set_play_lists_as_modified()

    def remove(self, p_rows):
        """Remove the given rows from the list."""
        SelectionList.remove(self, p_rows)
        _session = Session()
        _session.set_play_lists_as_modified()

