# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk

from Page import Page
from Menu import Menu
from PlayListList import PlayListList
from PlayListRenamingPage import PlayListRenamingPage
from PlayListRemovingPage import PlayListRemovingPage
from Session import Session


class PlayListPage(Page):
    """A page for managing a play list."""

    __list = None

    def __init__(self, p_play_list):
        Page.__init__(self)
        self.__list = PlayListList(p_play_list)
        self.__list.show()
        self.add(self.__list)

    def get_play_list(self):
        """Return the play list."""
        return self.__list.get_play_list()

    def update(self):
        """Update the list."""
        self.__list.update()

    def get_title(self):
        """Return the play list title."""
        return self.get_play_list().get_title()

    def get_menu(self):
        """Return the popup menu."""
        _menu = Menu()
        _menu.append_item('Rename...', self.__rename)
        _menu.append_item('Remove...', self.__remove)
        _menu.append_separator()
        _menu = self.__list.append_menu(_menu)
        return _menu

    def get_status(self):
        """Return the status message."""
        return self.__list.get_status()

    ###########################################################################
    # CALLBACKS
    ###########################################################################

    def __rename(self, p_menu_item):
        """Open a page for renaming the play list."""
#        print '__rename', self, p_menu_item
        _page = PlayListRenamingPage(self.get_play_list())
        _session = Session()
        _session.get_main_window().add_tab(_page)
        
    def __remove(self, p_menu_item):
        """Open a page for removing the play list."""
#        print '__remove', self, p_menu_item
        _page = PlayListRemovingPage(self.get_play_list())
        _session = Session()
        _session.get_main_window().add_tab(_page)

