# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk

from base.Manager import Manager

from Page import Page
from SimplePageButtonBox import SimplePageButtonBox
from Session import Session


class PlayListRemovingPage(Page):
    """A page for removing a play list."""

    __play_list = None

    __label = None

    def __init__(self, p_play_list):
        Page.__init__(self)
        self.__play_list = p_play_list
        # creation of a first line
        self.__label = gtk.GtkLabel('')
        self.__label.show()
        self.pack(self.__label, gtk.FALSE)
        # creation of a second line
        _box = SimplePageButtonBox(self.__apply, self.__cancel)
        _box.show()
        self.pack(_box, gtk.FALSE)
        # initialize the label
#        self.update()

    def get_play_list(self):
        """Return the play list to remove."""
        return self.__play_list

    def update(self):
        """Update the page."""
        Page.update(self)
        _text = 'Do you really remove the play list "%s" ?' % self.__play_list.get_title()
        self.__label.set_text(_text)

    def get_title(self):
        """Return the title of the page."""
        return 'Play list removing'

    ###########################################################################
    # CALLBACKS
    ###########################################################################

    def __apply(self):
        """Remove the play list, set play lists as modified
        and close all pages relative to the play list."""
#        print '__apply', self
        _manager = Manager()
        _set = _manager.get_play_lists_set()
        _set.remove(self.__play_list)
        _session = Session()
        _session.set_play_lists_as_modified()
        # close all pages which have the removed play list
        from PlayListPage import PlayListPage
        from PlayListRenamingPage import PlayListRenamingPage
        _len = _session.get_main_window().get_pages_number()
        _list = range(_len)
        _list.reverse()
        for _num in _list:
            _page = _session.get_main_window().get_page(_num)
            try:
                raise _page
            except PlayListPage:
                if _page.get_play_list() is self.__play_list:
                    _session.get_main_window().close_page(_num)
            except PlayListRenamingPage:
                if _page.get_play_list() is self.__play_list:
                    _session.get_main_window().close_page(_num)
            except PlayListRemovingPage:
                if _page.get_play_list() is self.__play_list:
                    _session.get_main_window().close_page(_num)
            except Page:
                pass
    
    def __cancel(self):
        """Close the page."""
#        print '__cancel', self
        _session = Session()
        _session.get_main_window().close_current_page()

    
    
