# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk

from Page import Page
from PlayListTitleFrame import PlayListTitleFrame
from SimplePageButtonBox import SimplePageButtonBox
from Session import Session


class PlayListRenamingPage(Page):
    """A page for renaming a play list."""

    __play_list = None

    __title_frame = None

    def __init__(self, p_play_list):
        Page.__init__(self)
        self.__play_list = p_play_list
        # creation of a first line
        self.__title_frame = PlayListTitleFrame('Rename a play list')
        self.__title_frame.show()
        self.pack(self.__title_frame, gtk.FALSE)
        # creation of a second line
        _box = SimplePageButtonBox(self.__apply, self.__cancel)
        _box.show()
        self.pack(_box, gtk.FALSE)
        # update the frame
        self.update()

    def get_play_list(self):
        """Return the play list to rename."""
        return self.__play_list

    def update(self):
        """Update the page."""
        Page.update(self)
        self.__title_frame.set_title(self.__play_list.get_title())

    def get_title(self):
        """Return the title of the page."""
        return 'Play list renaming'

    ###########################################################################
    # CALLBACKS
    ###########################################################################

    def __apply(self):
        """Rename the play list, set play lists as modified
        and update all labels of the notebook."""
#        print '__apply', self
        _new_title = self.__title_frame.get_title()
        self.__play_list.set_title(_new_title)
        _session = Session()
        _session.set_play_lists_as_modified()
        _session.get_main_window().update_labels()
        _session.get_main_window().close_current_page()
    
    def __cancel(self):
        """Close the page."""
#        print '__cancel', self
        _session = Session()
        _session.get_main_window().close_current_page()
