# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk

from base.Manager import Manager

from Session import Session
from Page import Page
from SongFrame import SongFrame
from SelectionList import SelectionList
from Menu import Menu

class PlayQueuePage(Page):
    """A page for managing the play queue."""

    __song_frame = None

    __list = None

    __id = None

    def __init__(self):
        Page.__init__(self, 0)
        self.__id = None
        self.__song_frame = SongFrame('Current song', gtk.FALSE)
        self.__song_frame.show()
        self.pack(self.__song_frame, 0)
        _manager = Manager()
        self.__list = SelectionList(_manager.get_play_thread().get_queue())
        self.__list.show()
        self.pack(self.__list)
        _manager.get_play_thread().set_changed_callback(self.__queue_changed)

    def update(self):
        """Update the page."""
        """
        _ret = gtk.TRUE
        if self.is_displayed():
            # update the current song frame
            _manager = Manager()
            _file = _manager.get_play_thread().get_current_file()
            if _file is not None:
                self.__song_frame.set_song(_file.get_song())
            else:
                self.__song_frame.clear()
            # update the queue
            self.__list.update()
            # update the status message
            Page.update(self)
            # create a timeout if necessary
            if self.__id is None:
                self.__id = gtk.timeout_add(5000, self.update)
        else:
            # destroy the timeout
            _ret = gtk.FALSE
            self.__id = None
        return _ret
        """
        Page.update(self)
        # update the current song frame
        _manager = Manager()
        _file = _manager.get_play_thread().get_current_file()
        if _file is not None:
            self.__song_frame.set_song(_file.get_song())
        else:
            self.__song_frame.clear()
        # update the queue
        self.__list.update()

    def get_title(self):
        """Return the title of the page."""
        return 'Play queue'

    def get_menu(self):
        """Return the popup menu."""
        _menu = Menu()
        _menu.append_item('Play next', self.__play_next)
        _menu.append_item('Stop', self.__stop)
        _manager = Manager()
        _looping = _manager.get_play_thread().get_looping()
        if _looping:
            _menu.append_item('Disable looping', self.__disable_looping)
        else:
            _menu.append_item('Enable looping', self.__enable_looping)
        _menu.append_separator()
#        _menu = ListedSongsMenu(_manager.get_play_thread().get_queue())
        _menu = self.__list.append_menu(_menu)
        return _menu

    def get_status(self):
        """Return the status message."""
        return self.__list.get_status()

    ###########################################################################
    # CALLBACKS
    ###########################################################################

    def __queue_changed(self):
        """Update the page."""
#        print '__queue_changed', self
        gtk.idle_add(self.__update)

    def __update(self):
        """Update the page and the status message (if the list is
        currently displayed)."""
#        print '__update', self
        self.update()
        if self.is_displayed():
            _session = Session()
            _session.get_main_window().set_status(self.get_status())

    def __play_next(self, p_menu_item):
        """Play the next song."""
#        print '__play_next', self, p_menu_item
        _manager = Manager()
        _manager.get_play_thread().play_next()
#        self.update()
        
    def __stop(self, p_menu_item):
        """Stop the playing."""
#        print '__stop', self, p_menu_item
        _manager = Manager()
        _manager.get_play_thread().stop()
#        self.update()

    def __enable_looping(self, p_menu_item):
        """Enable looping."""
#        print '__enable_looping', self, p_menu_item
        _manager = Manager()
        _manager.get_play_thread().set_looping(1)

    def __disable_looping(self, p_menu_item):
        """Disable looping."""
#        print '__disable_looping', self, p_menu_item
        _manager = Manager()
        _manager.get_play_thread().set_looping(0)

