# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk

from base.Manager import Manager

from Page import Page
from WizardPageButtonBox import WizardPageButtonBox
from SimplePageButtonBox import SimplePageButtonBox
from Menu import Menu
from SongFrame import SongFrame
from PathConfigFrame import PathConfigFrame
from Session import Session

class SongEditingPage(Page):
    """A page for editing a selection of songs."""

    __selection = None

    __song_frame = None

    __path_config_frame = None

    __button_box = None

    def __init__(self, p_selection):
        Page.__init__(self)
        self.__selection = p_selection
        # add a song frame
        self.__song_frame = SongFrame('', gtk.TRUE, self.__song_changed)
        self.__song_frame.show()
        self.pack(self.__song_frame, gtk.FALSE)
        # add a path config frame
        self.__path_config_frame = PathConfigFrame('Path configuration', self.__path_config_changed)
        _manager = Manager()
        _config = _manager.get_path_config()
        self.__path_config_frame.set_path_config(_config)
        self.__path_config_frame.show()
        self.pack(self.__path_config_frame, gtk.FALSE)
        # add some buttons
        self.__button_box = WizardPageButtonBox(self.__apply, self.__cancel, self.__cancel_all)
        self.__button_box.show()
        self.pack(self.__button_box, gtk.FALSE)
        self.__init()



    """
    def update(self):
        Page.update(self)
    """     

    def get_title(self):
        """Return the title of the page."""
        return 'Song editing'

    def __init(self):
        """Initialize the page to display the first song of the selection."""
        _file = self.__selection[0]
        self.__song_frame.set_label(_file.get_path())
        self.__song_frame.set_song(_file.get_song())
        if len(self.__selection) <= 1:
            self.__button_box.hide()
            self.__button_box.destroy()
            self.__button_box = SimplePageButtonBox(self.__apply, self.__cancel)
            self.__button_box.show()
            self.pack(self.__button_box, gtk.FALSE)

    def __view_next(self):
        """Remove the first song from the selection and reinitialize the page.

        If the selection becomes empty, the page is closing."""
        self.__selection = self.__selection[1:]
        if len(self.__selection) > 0:
            self.__init()
        else:
            _session = Session()
            _session.get_main_window().close_current_page()


    ###########################################################################
    # CALLBACKS
    ###########################################################################

    def __song_changed(self, p_new_song):
        """Update the example of the path configuration frame
        with the new song."""
#        print '__song_changed', self, p_new_song
        if self.__path_config_frame:
            self.__path_config_frame.update_example(p_new_song)

    def __path_config_changed(self, p_new_config):
        """Update the example of the path configuration frame
        with the new configuration."""
#        print '__path_config_changed', self, p_new_config
        _song = self.__song_frame.get_song()
        self.__path_config_frame.update_example(_song)

    def __cancel_all(self):
        """Close the page."""
#        print '__cancel_all', self
        _session = Session()
        _session.get_main_window().close_current_page()

    def __cancel(self):
        """Cancel the edition of the current song and pass to the next song."""
#        print '__cancel', self
        self.__view_next()

    def __apply(self):
        """Add a song editing task, set library and play lists as modified
        and pass to the next song."""
#        print '__apply', self
        # get mnager
        _manager = Manager()
        # save the current path config
        _config = self.__path_config_frame.get_path_config()
        _manager.set_path_config(_config)
        # add task for editing current song
        _file = self.__selection[0]
        _song = self.__song_frame.get_song()
        _task = _manager.get_song_editing(_file.get_path(), _song)
        _manager.get_tasks_thread().add_task(_task)
        _session = Session()
        _session.set_library_as_modified()
        _session.set_play_lists_as_modified()
        # pass to the next song file
        self.__view_next()
