# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk

from base.Manager import Manager
from base.Artist import Artist
from base.Disc import Disc
from base.Tune import Tune
from base.Song import Song
from base.Disc import DISC_SEP
from base.Artist import NAMES_SEP

from Widgets import Entry
from Widgets import Combo

class SongFrame(gtk.GtkFrame):
    """A frame for editing a song."""

    __title_entry = None

    __artist_entry = None

    __artist_combo = None

    __disc_entry = None

    __disc_combo = None

    __year_entry = None

    __track_entry = None

    __genre_entry = None

    __genre_combo = None

    __comment_entry = None

    __editable = gtk.TRUE

    __changed_callback = None

    def __init__(self, p_title, p_editable = gtk.TRUE, p_changed_callback = None):
        gtk.GtkFrame.__init__(self, p_title)
        self.__editable = p_editable
        self.__changed_callback = p_changed_callback
        _table = gtk.GtkTable(4, 4, gtk.FALSE)
        # add labels
        self.__add_label(_table, 'Title', 0, 0)
        self.__add_label(_table, 'Artist(s)', 1, 0)
        self.__add_label(_table, 'Disc', 2, 0)
        self.__title_entry = self.__add_entry('Tune title', _table, 0, 1)
        # add artist entry
        if self.__editable:
            self.__artist_combo = self.__add_combo('Artist names separated by ", "', _table, 1, 1)
            self.__artist_entry = self.__artist_combo.entry
        else:
            self.__artist_entry = self.__add_entry('Artist names separated by ", "', _table, 1, 1)
        # add disc entry
        if self.__editable:
            self.__disc_combo = self.__add_combo('Disc title followed, if needed, by " - " and the disc artist', _table, 2, 1)
            self.__disc_entry = self.__disc_combo.entry
        else:
            self.__disc_entry = self.__add_entry('Disc title followed, if needed, by " - " and the disc artist', _table, 2, 1)
        # add labels
        self.__add_label(_table, 'Year', 0, 2)
        self.__add_label(_table, 'Genre', 1, 2)
        self.__add_label(_table, 'Track', 2, 2)
        # add year entry
        self.__year_entry = self.__add_entry('Tune year', _table, 0, 3)
        # add genre entry
        if self.__editable:
            self.__genre_combo = self.__add_combo('Song genre', _table, 1, 3)
            self.__genre_entry = self.__genre_combo.entry
        else:
            self.__genre_entry = self.__add_entry('Song genre', _table, 1, 3)
        # add track entry
        self.__track_entry = self.__add_entry('Track number', _table, 2, 3)
        # add comment label
        self.__add_label(_table, 'Comment', 3, 0)
        # add comment entry
        self.__comment_entry = Entry(self.__editable, 'Comment', self.__changed)
        self.__comment_entry.show()
        _table.attach(self.__comment_entry, 1, 4, 3, 4)
        _table.set_row_spacings(3)
        _table.show()
        self.add(_table)
        # update lists
        gtk.idle_add(self.update_lists)


    def update_lists(self):
        """Update all combo boxes."""
        if self.__editable:
            self.update_artists_list()
            self.update_discs_list()
            self.update_genres_list()


    def __add_label(self, p_table, p_str, p_row, p_col):
        """Create a label with the given string,
        pack it into the given table at the given row and column
        and return it."""
        _label = gtk.GtkLabel(p_str)
        self.__add_widget(p_table, _label, p_row, p_col)
        return _label
    
    def __add_entry(self, p_help_message, p_table, p_row, p_col):
        """Create an entry, pack it into the given table
        at the given row and column and return it."""
        _entry = Entry(self.__editable, p_help_message, self.__changed)
        self.__add_widget(p_table, _entry, p_row, p_col)
        return _entry    

    def __add_combo(self, p_help_message, p_table, p_row, p_col):
        """Create a combo box, pack it into the given table
        at the given row and column and return it."""
        _combo = Combo(self.__editable, p_help_message, self.__changed)
        self.__add_widget(p_table, _combo, p_row, p_col)
        return _combo

    def __add_widget(self, p_table, p_widget, p_row, p_col):
        """Pack the given widget into the given table
        at the given row and column and return it."""
        p_widget.show()
        p_table.attach(p_widget, p_col, p_col + 1, p_row, p_row + 1)


    def set_song(self, p_song):
        """Display the given song

        If the given song is None, clear all entries."""
        if p_song is not None:
            self.__title_entry.set_text(p_song.get_tune().get_title())
            self.__artist_entry.set_text(str(p_song.get_artist()))
            self.__disc_entry.set_text(str(p_song.get_disc()))
            self.__year_entry.set_text(p_song.get_tune().get_year())
            self.__track_entry.set_text(p_song.get_track())
            self.__genre_entry.set_text(p_song.get_genre())
            self.__comment_entry.set_text(p_song.get_comment())
        else:
            self.clear()

    def clear(self):
        """Clear all entries."""
        self.__title_entry.set_text('')
        self.__artist_entry.set_text('')
        self.__disc_entry.set_text('')
        self.__year_entry.set_text('')
        self.__track_entry.set_text('')
        self.__genre_entry.set_text('')
        self.__comment_entry.set_text('')
        


    def get_song(self):
        """Return the entered song."""
        # get title
        _title = self.__title_entry.get_text().strip()
        # get artist(s)
        _artist = None
        _artist_str = self.__artist_entry.get_text().strip()
        _list = _artist_str.split(NAMES_SEP)
        if len(_list) >= 1:
            _artist = Artist(_list[0])
            for _name in _list[1:]:
                _artist.add_name(_name)
        # get disc
        _disc = self.__disc_entry.get_text()
        _list = _disc.split(DISC_SEP)
        if len(_list) == 2:
            _artist_name = _list[1].strip()
            _disc_artist = Artist(_artist_name)
            _disc_title = _list[0].strip()
            _disc = Disc(_disc_title, _disc_artist)
        else:
            _disc_title = _disc.strip()
            _disc = Disc(_disc_title)
        # get year
        _year = self.__year_entry.get_text().strip()
        # get track
        _track = self.__track_entry.get_text().strip()
        # get genre
        _genre = self.__genre_entry.get_text().strip()
        # get comment
        _comment = self.__comment_entry.get_text().strip()
        # create tune
        _tune = Tune(_title, _year)
        # create song
        _song = Song(_artist, _tune, _disc, _track, _genre, _comment)
        return _song

    def update_artists_list(self):
        """Update the artist combo list."""
        if self.__editable:
            # get the current artist
            _cur_artist = self.__artist_entry.get_text()
            # search for all artists
            _manager = Manager()
            _artists = _manager.get_library_selection().search_artist()
            # create a list of artists names
            _names = []
            for _art in _artists:
                _names.append(str(_art))
            # add the null string if needed
            if '' not in _names:
                _names.append('')
            # sort the list and set it to the artist combo
            _names.sort()
            self.__artist_combo.set_popdown_strings(_names)
            # set again the old artist
            self.__artist_entry.set_text(_cur_artist)

    def update_discs_list(self):
        """Update the disc combo list."""
        if self.__editable:
            # get the current artist
            _cur_disc = self.__disc_entry.get_text()
            # search for all discs
            _manager = Manager()
            _discs = _manager.get_library_selection().search_disc()
            _titles = []
            # create a list of titles of discs
            for _disc in _discs:
                _titles.append(str(_disc))
            # add the null string if needed
            if '' not in _titles:
                _titles.append('')
            # sort the list and set it to the disc combo
            _titles.sort()
            self.__disc_combo.set_popdown_strings(_titles)
            # set again the old disc
            self.__disc_entry.set_text(_cur_disc)

    def update_genres_list(self):
        """Update the genre combo list."""
        if self.__editable:
            # get the current genre
            _cur_genre = self.__genre_entry.get_text()
            # search for all genres
            _manager = Manager()
            _genres = _manager.get_library_selection().search_genre()
            # add the null string if needed
            if '' not in _genres:
                _genres.append('')
            # sort the list and set it to the genre combo
            _genres.sort()
            self.__genre_combo.set_popdown_strings(_genres)
            # set again the old genre
            self.__genre_entry.set_text(_cur_genre)

    def set_track(self, p_track):
        """Set the track."""
        self.__track_entry.set_text(str(p_track))
        


    ###########################################################################
    # CALLBACKS
    ###########################################################################

    def __changed(self, p_entry):
        """Call the callback function with the new song as argument."""
#        print '__changed', self, p_entry
        if self.__changed_callback:
            _song = self.get_song()
            self.__changed_callback(_song)
