# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk

from base.Manager import Manager

from Page import Page
from SimplePageButtonBox import SimplePageButtonBox
from WizardPageButtonBox import WizardPageButtonBox
from Session import Session


class SongRemovingPage(Page):
    """A page for removing a selection of song files from the library."""

    __selection = None

    __label = None

    __button_box = None

    def __init__(self, p_selection):
        Page.__init__(self)
        self.__selection = p_selection
        # creation of a first line
        self.__label = gtk.GtkLabel('')
        self.__label.show()
        self.pack(self.__label, gtk.FALSE)
        # creation of a second line
        self.__button_box = WizardPageButtonBox(self.__apply, self.__cancel, self.__cancel_all)
        self.__button_box.show()
        self.pack(self.__button_box, gtk.FALSE)
        # initialize the label
        self.__init()
        
    """
    def update(self):
        Page.update(self)
        _file = self.__selection[0]
        _text = 'Do you really remove the file "%s" ?' % _file.get_path()
        self.__label.set_text(_text)
    """

    def get_title(self):
        """Return the title of the page."""
        return 'Song removing'

    def __init(self):
        """Initialize the page to display the first song of the selection."""
        _file = self.__selection[0]
        _text = 'Do you really remove the file "%s" ?' % _file.get_path()
        self.__label.set_text(_text)
        if len(self.__selection) <= 1:
            self.__button_box.hide()
            self.__button_box.destroy()
            self.__button_box = SimplePageButtonBox(self.__apply, self.__cancel)
            self.__button_box.show()
            self.pack(self.__button_box, gtk.FALSE)

    def __view_next(self):
        """Remove the first song from the selection and reinitialize the page.

        If the selection becomes empty, the page is closing."""
        self.__selection = self.__selection[1:]
        if len(self.__selection) > 0:
            self.__init()
        else:
            _session = Session()
            _session.get_main_window().close_current_page()

    ###########################################################################
    # CALLBACKS
    ###########################################################################

    def __cancel_all(self):
        """Close the page."""
#        print '__cancel_all', self
        # close the page
        _session = Session()
        _session.get_main_window().close_current_page()


    def __cancel(self):
        """Cancel the edition of the current song and pass to the next song."""
#        print '__cancel', self
        # pass to the next song file
        self.__view_next()

    def __apply(self):
        """Add a song removing task, set library and play lists as modified
        and pass to the next song."""
#        print '__apply', self
        # add the song removing task to the tasks queue
        _manager = Manager()
        _path = self.__selection[0].get_path()
        _task = _manager.get_song_removing(_path)
        _manager.get_tasks_thread().add_task(_task)
        # set the libray and play lists as modified
        _session = Session()
        _session.set_library_as_modified()
        _session.set_play_lists_as_modified()
        # pass to the next song file
        self.__view_next()
    
