# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk

from base.Manager import Manager
from base.Task import Task
from base.MP3Importing import MP3Importing
from base.TrackImporting import TrackImporting
from base.SongAdding import SongAdding
from base.SongEditing import SongEditing
from base.SongRemoving import SongRemoving
from base.LibraryRefreshing import LibraryRefreshing
from base.LibraryLoading import LibraryLoading
from base.LibrarySaving import LibrarySaving
from base.PlayListsLoading import PlayListsLoading
from base.PlayListsSaving import PlayListsSaving
from base.PathConfigLoading import PathConfigLoading
from base.PathConfigSaving import PathConfigSaving

from List import List
from Menu import Menu


class TasksList(List):
    """A tasks queue."""

    def __init__(self):
        List.__init__(self, [], 1)
        self.set_widths([500])
        
    def get_tasks(self):
        """Return the tasks sequence."""
        return self.get_sequence()

    def set_tasks(self, p_sequence):
        """Set the tasks sequence."""
        self.set_sequence(p_sequence)

    def get_tuple(self, p_task):
        """Return the tuple corresponding to the given task."""
        _item = 'To do nothing good...'
        try:
            raise p_task
        except LibraryLoading, _task:
            _item = 'To load library from file %s' % _task.get_file_path()
        except LibrarySaving, _task:
            _item = 'To save library in file %s' % _task.get_file_path()
        except PlayListsLoading, _task:
            _item = 'To load play lists from directory %s' % _task.get_dir_path()
        except PlayListsSaving, _task:
            _item = 'To save play lists in file %s' % _task.get_dir_path()
        except PathConfigLoading, _task:
            _item = 'To load path configuration from file %s' % _task.get_file_path()
        except PathConfigSaving, _task:
            _item = 'To save path configuration in file %s' % _task.get_file_path()
        except SongAdding, _task:
            _item = 'To add the file %s' % _task.get_src_file_path()
        except MP3Importing, _task:
            _item = 'To convert, encode and add the file %s' % _task.get_src_file_path()
        except TrackImporting, _task:
            _item = 'To rip, encode and add the track %s' % _task.get_track()
        except SongEditing, _task:
            _item = 'To edit the file %s and replace it by the file %s' % (_task.get_old_file_path(), _task.get_new_file_path())
        except SongRemoving, _task:
            _item = 'To remove the file %s' % _task.get_file_path()
        except LibraryRefreshing, _task:
            _item = 'To refresh the library from the directory %s' % _task.get_root_path()
        except Task:
            _item = 'To execute an unknown task'
        return [_item]

    def get_selected_tasks(self):
        """Return the selected tasks."""
        return self.get_selected_items()


    def get_menu(self):
        """Return the popup menu."""
        _menu = Menu()
        _menu.append_item('Cancel', self.__cancel)
        return _menu

    def update(self):
        """Update the list."""
        _manager = Manager()
        _queue = _manager.get_tasks_thread().get_queue()
        self.set_sequence(_queue)
        List.update(self)

    def get_status(self):
        """Return the status message."""
        _ret = '%d tasks - %d selected tasks' % (len(self.get_sequence()), len(self.get_selected_items()))
        return _ret

    ###########################################################################
    # CALLBACKS
    ###########################################################################

    def __cancel(self, p_menu_item):
        """Cancel the selected tasks."""
#        print '__cancel', self, p_menu_item
        _manager = Manager()
        _thread = _manager.get_tasks_thread()
        # get a sorted rows list
        _rows = self.get_selected_rows()
        _rows.sort()
        _rows.reverse()
        # cancel all selected tasks, beginning by the end
        for _row in _rows:
            _thread.cancel_task(_row)
        # update list
        self.update()

