# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk

from util.UserException import UserException
from util.File import FileNotFoundException
from util.File import FileAlreadyExistsException
from util.File import FileNotReadableException
from util.File import FileNotWritableException
from util.Directory import DirectoryNotFoundException
from util.Directory import DirectoryAlreadyExistsException
from util.Directory import DirectoryNotReadableException
from util.Directory import DirectoryNotEmptyException
from util.Directory import DirectoryNotWritableException
from util.Directory import DirectoryNotAccessibleException

from base.Process import BadStatusException
from base.MusicFile import FileNotOggVorbisException
from base.MusicFile import FileNotMP3Exception
from base.LibraryLoading import LibraryParserException
from base.Manager import Manager

from Page import Page
from List import List
from Menu import Menu
from TasksList import TasksList
from Session import Session


class TasksQueuePage(Page):
    """A tasks thread managing page."""

    __list = None

    __errors_list = None

    def __init__(self):
        Page.__init__(self, 0)
        self.__list = TasksList()
        self.__list.show()
        self.pack(self.__list, gtk.TRUE)
        self.__errors_list = List([], 1)
        self.__errors_list.set_widths([500])
        self.__errors_list.get_tuple = self.__get_error_tuple
        self.__errors_list.show()
        self.pack(self.__errors_list, gtk.TRUE)
        # set a callback on the change of the tasks queue
        _manager = Manager()
        _manager.get_tasks_thread().set_changed_callback(self.__queue_changed)

    def __get_error_tuple(self, p_error):
        """Return the tuple corresponding to the given exception."""
        _ret = 'A very bad error...'
        try:
            raise p_error
        except FileNotFoundException, _exc:
            _ret = 'The file %s does not exist' % _exc
        except FileAlreadyExistsException, _exc:
            _ret = 'The file %s does already exist' % _exc
        except FileNotReadableException, _exc:
            _ret = 'The file %s cannot be read' % _exc
        except FileNotWritableException, _exc:
            _ret = 'The file %s cannot be written' % _exc
        except DirectoryNotFoundException, _exc:
            _ret = 'The directory %s does not exist' % _exc
        except DirectoryAlreadyExistsException, _exc:
            _ret = 'The directory %s does already exist' % _exc
        except DirectoryNotReadableException, _exc:
            _ret = 'The directory %s cannot be read' % _exc
        except DirectoryNotEmptyException, _exc:
            _ret = 'The directory %s is not empty' % _exc
        except DirectoryNotWritableException, _exc:
            _ret = 'The directory %s cannot be written' % _exc
        except DirectoryNotAccessibleException, _exc:
            _ret = 'The directory %s cannot be accessed' % _exc
        except BadStatusException, _exc:
            _ret = 'A process results a bad status %s' % _exc
        except FileNotOggVorbisException, _exc:
            _ret = 'The file %s is not an Ogg Vorbis file' % _exc
        except FileNotMP3Exception, _exc:
            _ret = 'The file %s is not a MP3 file' % _exc
        except LibraryParserException, _exc:
            _ret = 'A XML parse error occured in file %s at line %s and column %s : %s - %s' % (_exc.get_file_path(), _exc.get_line(), _exc.get_column(), _exc.get_code(), _exc.get_message())
        except UserException, _exc:
            _ret = 'An unknown error occured : %s' % _exc
        return [_ret]


    def update(self):
        """Update the page."""
        # update the current tasks list
        self.__list.update()
        # update the errors list
        _manager = Manager()
        _errors = _manager.get_tasks_thread().get_errors()
        self.__errors_list.set_sequence(_errors)
        self.__errors_list.update()
        # update the page
        Page.update(self)
        
    def get_title(self):
        """Return the title of the page."""
        return 'Tasks queue'

    def get_menu(self):
        """Return the popup menu."""
        return self.__list.append_menu()

    def get_status(self):
        """Return the status message."""
        return self.__list.get_status()


    ###########################################################################
    # CALLBACKS
    ###########################################################################

    def __queue_changed(self):
        """Update the page."""
#        print '__queue_changed', self
        gtk.idle_add(self.__update)

    def __update(self):
        """Update the page and the status message (if the list is
        currently displayed)."""
#        print '__update', self
        self.update()
        if self.is_displayed():
            _session = Session()
            _session.get_main_window().set_status(self.get_status())

