# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk

from Widgets import SpinButton

class TrackFrame(gtk.GtkFrame):
    """A frame for editing a track."""

    __track_spin_button = None

    __changed_callback = None

    def __init__(self, p_title, p_changed_callback = None):
        gtk.GtkFrame.__init__(self, p_title)
        self.__changed_callback = p_changed_callback
        _box = gtk.GtkHBox(gtk.FALSE, 10)
        # add a label
        _label = gtk.GtkLabel('Track number')
#        _label.set_justify(gtk.JUSTIFY_LEFT)
        _label.show()
        _box.pack_start(_label, gtk.FALSE, gtk.FALSE, 20)
        # add a spin button
        _adj = gtk.GtkAdjustment(0, 1, 99, 1, 10)
        self.__track_spin_button = SpinButton(_adj, 'CDDA track number', self.__changed)
        self.__track_spin_button.set_numeric(gtk.TRUE)
        self.__track_spin_button.show()
        _box.pack_start(self.__track_spin_button, gtk.FALSE)
        # add the box
        _box.show()
        self.add(_box)

    def get_track(self):
        """Return the entered track."""
        return self.__track_spin_button.get_value_as_int()
        
    def set_track(self, p_track_number):
        """Set the given track number."""
        self.__track_spin_button.set_value(p_track_number)

    ###########################################################################
    # CALLBACKS
    ###########################################################################


    def __changed(self, p_spin_button):
        """Call the callback function with the new track as argument."""
#        print '__changed', self, p_spin_button
        if self.__changed_callback:
            _track = self.get_track()
            self.__changed_callback(_track)
        
