# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk

from base.Manager import Manager

from Page import Page
from SelectionList import SelectionList
from Menu import Menu
from TrackFrame import TrackFrame
from SongFrame import SongFrame
from PathConfigFrame import PathConfigFrame
from SimplePageButtonBox import SimplePageButtonBox
from Session import Session

last_track = 0

last_song = None

class TrackImportingPage(Page):
    """A page for importing a track in the library."""

    __track_frame = None

    __song_frame = None

    __path_config_frame = None

    def __init__(self):
        Page.__init__(self)
        # add a track frame
        self.__track_frame = TrackFrame('Track', self.__track_changed)
        self.__track_frame.show()
        self.pack(self.__track_frame, gtk.FALSE)
        # add a song frame
        self.__song_frame = SongFrame('Song', gtk.TRUE, self.__song_changed)
        self.__song_frame.show()
        self.pack(self.__song_frame, gtk.FALSE)
        # add a path config frame
        self.__path_config_frame = PathConfigFrame('Path configuration', self.__path_config_changed)
        self.__path_config_frame.show()
        self.pack(self.__path_config_frame, gtk.FALSE)
        # add buttons
        _button_box = SimplePageButtonBox(self.__apply, self.__cancel)
        _button_box.show()
        self.pack(_button_box, gtk.FALSE)
        # initialize the path config frame
        _manager = Manager()
        _config = _manager.get_path_config()
        self.__path_config_frame.set_path_config(_config)
        # initialize the song frame with a blank title
        if last_song is not None:
            _song = last_song.clone()
            _song.get_tune().set_title('')
            self.__song_frame.set_song(_song)
        # initialize the track frame with an incremented track number
        self.__track_frame.set_track(last_track + 1)

    """
    def update(self):
        Page.update(self)
    """

    def get_title(self):
        """Return the title of the page."""
        return 'Track importing'


    ###########################################################################
    # CALLBACKS
    ###########################################################################

    def __track_changed(self, p_new_track):
        """Set the new track in the song frame."""
        self.__song_frame.set_track(p_new_track)

    def __song_changed(self, p_new_song):
        """Update the example path configuration with the given song."""
#        if self.__path_config_frame:
        self.__path_config_frame.update_example(p_new_song)

    def __path_config_changed(self, p_new_config):
        """Update the example path configuration
        with the given configuration."""
        _song = self.__song_frame.get_song()
        self.__path_config_frame.update_example(_song)

    def __cancel(self):
        """Close the page."""
#        print '__cancel', self
        # close the page
        _session = Session()
        _session.get_main_window().close_current_page()

    def __apply(self):
        """Add the track importing task, set the library as modified
        and close the page."""
        global last_track, last_song
#        print '__apply', self
        _manager = Manager()
        # set the path config
        _config = self.__path_config_frame.get_path_config()
        _manager.set_path_config(_config)
        # get track and song
        last_track = self.__track_frame.get_track()
        last_song = self.__song_frame.get_song()
        # get the track importing task
        _task = _manager.get_track_importing(last_track, last_song)
        # add the task to the tasks thread
        _manager.get_tasks_thread().add_task(_task)
        # set the library as modified and close the page
        _session = Session()
        _session.set_library_as_modified()
        _session.get_main_window().close_current_page()
