# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk

from Session import Session

def connect_help(p_widget, p_help_message):
    p_widget.connect('focus-in-event', __focus_in, p_help_message)
    p_widget.connect('focus-out-event', __focus_out)
    

def connect_to_changes(p_widget, p_signal, p_callback):
    """Connect the changed event to the given callback."""
    if p_callback:
        p_widget.connect(p_signal, p_callback)

###########################################################################
# CALLBACKS
###########################################################################

def __focus_out(p_widget, p_event):
    """Delete the status bar message."""
#    print '__focus_out', p_widget, p_event
    _session = Session()
    _session.get_main_window().set_status('')

def __focus_in(p_widget, p_event, p_message):
    """Display a help message in status bar."""
#    print '__focus_in', p_widget, p_event, p_message
    _session = Session()
    _session.get_main_window().set_status(p_message)


class Entry(gtk.GtkEntry):
    """A specific entry."""

    def __init__(self, p_editable = gtk.TRUE, p_help_message = '', p_changed_callback = None):
        gtk.GtkEntry.__init__(self)
        self.set_editable(p_editable)
        connect_help(self, p_help_message)
        connect_to_changes(self, 'changed', p_changed_callback)

class Combo(gtk.GtkCombo):
    """A specific combo."""

    def __init__(self, p_editable = gtk.TRUE, p_help_message = '', p_changed_callback = None):
        gtk.GtkCombo.__init__(self)
        self.entry.set_editable(p_editable)
        connect_help(self.entry, p_help_message)
        connect_to_changes(self.entry, 'changed', p_changed_callback)

class CheckButton(gtk.GtkCheckButton):
    """A specific check button."""

    def __init__(self, p_label, p_help_message = '', p_changed_callback = None):
        gtk.GtkCheckButton.__init__(self, p_label)
        connect_help(self, p_help_message)
        connect_to_changes(self, 'toggled', p_changed_callback)


class SpinButton(gtk.GtkSpinButton):
    """A specific spin button."""

    def __init__(self, p_adjustement, p_help_message = '', p_changed_callback = None):
        gtk.GtkSpinButton.__init__(self, p_adjustement, 0.5, 0)
        connect_help(self, p_help_message)
        connect_to_changes(self, 'changed', p_changed_callback)

