# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import sys

try:
    import gtk
except ImportError:
    sys.stderr.write("Sorry but maestro-gui requires PyGTK+\n")
    sys.exit(1)
    
from base.Manager import Manager
from base.PlayList import PlayList
from base.SongFile import SongFile
from base.Song import get_empty_song

from gui.Session import Session
from gui.PlayListPage import PlayListPage
from gui.LibraryPage import LibraryPage
from gui.TasksQueuePage import TasksQueuePage
from gui.PlayQueuePage import PlayQueuePage
from gui.MainWindow import MainWindow


if __name__ == "__main__":
    _manager = Manager()
#    print "Starting play thread..."
    _manager.get_play_thread().start()
#    print "Starting tasks thread..."
    _manager.get_tasks_thread().start()

#    print 'Loading path configuration...'
    _task = _manager.get_path_config_loading()
    _manager.get_tasks_thread().add_task(_task)

#    print 'Loading library...'
    _task = _manager.get_library_loading()
    _manager.get_tasks_thread().add_task(_task)

#    print 'Loading play lists...'
    _task = _manager.get_play_lists_loading()
    _manager.get_tasks_thread().add_task(_task)

#    print "Show main window..."
    _main_win = MainWindow()
    _main_win.show()
    _session = Session(_main_win)

    # display the library page
    _page = LibraryPage()
    _main_win.add_tab(_page)
    
    # display the play queue page
    _page = PlayQueuePage()
    _main_win.add_tab(_page)

    # display the tasks queue page
    _page = TasksQueuePage()
    _main_win.add_tab(_page)

    gtk.mainloop()

    _main_win.hide()
#    print "Killing tasks thread..."
    _manager.get_tasks_thread().kill()
#    print "Killing play thread..."
    _manager.get_play_thread().kill()

