# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from util.Directory import DirectoryNotFoundException
from util.Directory import DirectoryNotAccessibleException
from util.File import FileNotFoundException
from util.File import FileNotReadableException

from base.Manager import Manager
from base.LibraryLoading import LibraryParserException

from text.Session import Session
from text.AboutMessage import AboutMessage
from text.MainMenu import MainMenu

def print_debug_info():
    _manager = Manager()
    # get the library
    _lib = _manager.get_library_selection()
    print
    _list = _lib.search_artist()
    for _item in _list:
        print repr(_item), "-->", _item
    _list = _lib.search_disc()
    print
    for _item in _list:
        print repr(_item), "-->", _item
    print
    _list = _lib.search_tune()
    for _item in _list:
        print repr(_item), "-->", _item

    
def print_about():
    _msg = AboutMessage()
    print _msg


if __name__ == "__main__":
    # display some useful information
    print_about()
    # get the manager
    _manager = Manager()
    # load the path configuration
    print "Loading the path configuration..."
    _task = _manager.get_path_config_loading()
    try:
        _task.start()
    except FileNotFoundException, _exc:
        print 'The file %s does not exist. Loading default path configuration...' % _exc
    except FileNotReadableException, _exc:
        print 'The file %s is not readable. Loading default path configuration...' % _exc
    # load the library
    print "Loading the library..."
    _task = _manager.get_library_loading()
    try:
        _task.start()
    except FileNotFoundException, _exc:
        print 'The file %s does not exist. Loading an empty library...' % _exc
    except FileNotReadableException, _exc:
        print 'The file %s is not readable. Loading an empty library...' % _exc
    except LibraryParserException, _exc:
        print 'An error occured while reading the file %s at line %d and column %d : %d - %s' % (_exc.get_file_path(), _exc.get_line(), _exc.get_column(), _exc.get_code(), _exc.get_message())
        print 'Loading an empty library...'
    # load play lists
    print "Loading play lists..."
    _manager.get_play_lists_loading().start()
    # load the play thread
    print "Starting the play thread..."
    _manager.get_play_thread().start()
    # load the tasks thread
    print "Starting the tasks thread..."
    _manager.get_tasks_thread().start()
    # diplay debug information
#    print_debug_info()
    # displays the main menu
    try:
        _menu = MainMenu()
        _menu.show()
    finally:
        # stop the tasks thread
        print "Stoping the tasks thread..."
        _thread = _manager.get_tasks_thread()
        _thread.kill()
        _thread.join()
        # stop the play thread
        print "Stoping the play thread..."
        _thread = _manager.get_play_thread()
        _thread.kill()
        _thread.join()
        # save all play lists
        if Session.play_lists_modified:
            print "Saving play lists..."
            _task = _manager.get_play_lists_saving()
            _task.start()
        # save the library
        if Session.library_modified:
            print "Saving the library..."
            _task = _manager.get_library_saving()
            _task.start()
        # save the path configuration
        print "Saving the path configuration..."
        _task = _manager.get_path_config_saving()
        _task.start()

