# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


from util.Directory import DirectoryNotFoundException
from util.Directory import DirectoryNotReadableException
from util.File import FileNotFoundException
from util.File import FileNotReadableException

from base.Manager import Manager
from base.Song import Song
from base.Song import get_empty_song
from base.Disc import Disc
from base.MusicDirectory import MusicDirectory
from base.MusicFile import FileNotOggVorbisException
from base.Process import ToolNotFoundException
from base.TagsReading import TagsReading
from base.SongAdding import get_song_adding_tools
from base.SongAdding import can_add_song
from base.MP3Importing import get_mp3_importing_tools
from base.MP3Importing import can_import_mp3
from base.TrackImporting import get_track_importing_tools
from base.TrackImporting import can_import_track

from Session import Session
from Menu import Menu
from Question import Question
from NumberQuestion import NumberQuestion
from Edition import Edition
from MissingToolsMessage import MissingToolsMessage


class AddMenu(Menu):
    "Menu which allows to add a song."

    __last_ogg_file_path = ''

    __last_ogg_dir_path = ''

    __last_mp3_file_path = ''

    __last_mp3_dir_path = ''

    __last_track_number = 1

    def __init__(self):
        "Constructs the menu."
        Menu.__init__(self, "Add songs to the library", "Your choice")
        self.add_entry("Copy an Ogg Vorbis file", self.__add_file)
        self.add_entry("Copy an Ogg Vorbis files from a directory", self.__add_file_dir)
        self.add_entry("Convert a MP3 file", self.__add_mp3)
        self.add_entry("Convert MP3 files from a directory", self.__add_mp3_dir)
        self.add_entry("Rip and encode a CDDA track", self.__add_cdda_track)
        self.add_entry("Rip and encode some CDDA tracks", self.__add_cdda_all)
        self.add_entry("Go back", self.go_back)


    def __edit_song(self, p_song):
#        _ovfm = Session.ovfm
        _edit = Edition(p_song)
        _edit.show()
        _song = _edit.get_song()
        return _song

    def __ask_for_disc(self):
        # TO DO : prendre le nom du repertoire pour le titre du disque
        _disc = Disc("")
        _song = Song([], None, _disc)
        _edition = Edition(_song)
        if not _edition.ask_for_disc():
            # asks if the disc is an album
            _is_album = _edition.ask_for_an_album()
            # asks for singers
            if _is_album:
                _edition.ask_for_singers()
            # asks for disc title
            _edition.ask_for_disc_title()
            # updates the disc singer
            if _is_album:
                _edition.set_disc_as_album()
        # returns a disc
        _disc = _edition.get_song().get_disc()
        return _disc

    def __set_disc_to_song(self, p_song, p_disc):
        _song = p_song
        # changes the disc
        _song.set_disc(p_disc)
        """
        # adds the disc artist as singer
        if p_disc.has_artist():
            _artist = p_disc.get_artist()
            _singers = _song.get_singers()
            if _artist not in _singers:
                _singers.insert(0, _artist)
                _song.set_singers(_singers)
        """
        return _song

    def __add_file(self):
        if not can_add_song():
            _error = 'Sorry but you cannot import an Ogg Vorbis file.'
            _tools = get_song_adding_tools()
            _msg = MissingToolsMessage(_error, _tools)
            _msg.display()
        else:
            _manager = Manager()
            # ask for path
            _quest = Question('File path', self.__last_ogg_file_path)
            self.__last_ogg_file_path = _quest.show()
            # reads file tags
            _task = TagsReading(self.__last_ogg_file_path)
            try:
                _song = _task.start()
            except ToolNotFoundException, _exc:
                print "Error : the tool %s is not available on your system" % _exc
            except FileNotFoundException, _exc:
                print "Error : the file %s does not exist." % _exc
            except FileNotReadableException, _exc:
                print "Error : the file %s is not readable." % _exc
            except FileNotOggVorbisException, _exc:
                print "Error : the file %s is not an Ogg Vorbis file." % _exc
            else:
                # edits the song
                _song = self.__edit_song(_song)
                # adds the new Ogg Vorbis file
                _task = _manager.get_song_adding(self.__last_ogg_file_path, _song)
                _manager.get_tasks_thread().add_task(_task)
                print "The Ogg Vorbis file %s will be added to the library" % self.__last_ogg_file_path
                Session.library_modified = 1
        return 1

    def __add_file_dir(self):
        if not can_add_song():
            _error = 'Sorry but you cannot import an Ogg Vorbis files directory.'
            _tools = get_song_adding_tools()
            _msg = MissingToolsMessage(_error, _tools)
            _msg.display()
        else:
            _manager = Manager()
            _quest = Question("Directory path", self.__last_ogg_dir_path)
            self.__last_ogg_dir_path = _quest.show()
            _dir = MusicDirectory(self.__last_ogg_dir_path)
            try:
                _list = _dir.list_ogg_vorbis_files()
            except DirectoryNotFoundException, _exc:
                print "Error : the directory does not exist : %s" % _exc
            except DirectoryNotReadableException, _exc:
                print "Error : the directory is not readable : %s" % _exc
            else:
                if len(_list) == 0:
                    print "No Ogg Vorbis file found in the directory."
                else:
                    _disc = self.__ask_for_disc()
                    for _file in _list:
                        print "*** %s ***" % _file.get_path()
                        # reads file tags
                        _task = TagsReading(_file.get_path())
                        try:
                            _song = _task.start()
                        except ToolNotFoundException, _exc:
                            print "Error : the tool %s is not available on your system" % _exc
                        except FileNotReadableException, _exc:
                            print "Error : the file is not readable : %s" % _exc
                        else:
                            # changes the disc and the artist if necessary
                            _song = self.__set_disc_to_song(_song, _disc)
                            # edits the song
                            _song = self.__edit_song(_song)
                            # adds the new Ogg Vorbis file
                            _task = _manager.get_song_adding(_file.get_path(), _song)
                            _manager.get_tasks_thread().add_task(_task)
                            print "The Ogg Vorbis file %s will be added to the library" % _file.get_path()
                            Session.library_modified = 1
        return 1

    def __add_mp3(self):
        if not can_import_mp3():
            _error = 'Sorry but you cannot import a MP3 file.'
            _tools = get_mp3_importing_tools()
            _msg = MissingToolsMessage(_error, _tools)
            _msg.display()
        else:
            _manager = Manager()
            _quest = Question('File path', self.__last_mp3_file_path)
            self.__last_mp3_file_path = _quest.show()
            # edit the song
            _song = get_empty_song()
            _song = self.__edit_song(_song)
            # convert the MP3 file and adds the new Ogg Vorbis file
            _task = _manager.get_mp3_importing(self.__last_mp3_file_path, _song)
            _manager.get_tasks_thread().add_task(_task)
            print "The MP3 file %s will be added to the library" % self.__last_mp3_file_path
            Session.library_modified = 1
        return 1

    def __add_mp3_dir(self):
        if not can_import_mp3():
            _error = 'Sorry but you cannot import a MP3 files directory.'
            _tools = get_mp3_importing_tools()
            _msg = MissingToolsMessage(_error, _tools)
            _msg.display()
        else:
            _manager = Manager()
            _quest = Question("Directory path", self.__last_mp3_dir_path)
            self.__last_mp3_dir_path = _quest.show()
            _dir = MusicDirectory(self.__last_mp3_dir_path)
            try:
                _list = _dir.list_mp3_files()
            except DirectoryNotFoundException, _exc:
                print "Error : the directory does not exist : %s" % _exc
            except DirectoryNotReadableException, _exc:
                print "Error : the directory is not readable : %s" % _exc
            else:
                # stop if the list is null
                if len(_list) == 0:
                    print "No MP3 file found in the directory."
                else:
                    _disc = self.__ask_for_disc()
                    _song = get_empty_song()
                    # change the disc and the artist if necessary
                    _song = self.__set_disc_to_song(_song, _disc)
                    for _file in _list:
                        print "*** %s ***" % _file.get_path()
                        # edit a song
                        _song = self.__edit_song(_song)
                        # convert the MP3 file and add the new Ogg Vorbis file
                        _task = _manager.get_mp3_importing(_file.get_path(), _song)
                        _manager.get_tasks_thread().add_task(_task)
                        print "The file %s will be added to the library" % _file.get_path()
                        _song = _song.clone()
                        Session.library_modified = 1
        return 1

    def __add_cdda_track(self):
        if not can_import_track():
            _error = 'Sorry but you cannot import a CDDA track.'
            _tools = get_track_importing_tools()
            _msg = MissingToolsMessage(_error, _tools)
            _msg.display()
        else:
            _manager = Manager()
            _quest = NumberQuestion("Track number", self.__last_track_number)
            _track = _quest.show()
            # update the last track number attribute
            self.__last_track_number = _track
            # edits the song
            _song = get_empty_song()
            _song.set_track(str(_track))
            _song = self.__edit_song(_song)
            # rips the CDDA track and adds the new Ogg Vorbis file
            _task = _manager.get_track_importing(_track, _song)
            _manager.get_tasks_thread().add_task(_task)
            print "The track %d will be added to the library" % _track
            Session.library_modified = 1
        return 1

    def __add_cdda_all(self):
        if not can_import_track():
            _error = 'Sorry but you cannot import a CDDA track.'
            _tools = get_track_importing_tools()
            _msg = MissingToolsMessage(_error, _tools)
            _msg.display()
        else:
            _manager = Manager()
            _quest = NumberQuestion("First track", self.__last_track_number)
            _start = _quest.show()
            _quest = NumberQuestion("Last track", _start)
            _stop = _quest.show()
            _song = get_empty_song()
            for _track in range(_start, _stop + 1):
                print "*** Track %d ***" % _track
                # edits the song
                _song.set_track(str(_track))
                _song = self.__edit_song(_song)
                # rips the CDDA track and adds the new Ogg Vorbis file
                _task = _manager.get_track_importing(_track, _song)
                _manager.get_tasks_thread().add_task(_task)
                print "The track %d will be added to the library" % _track
                _song = _song.clone()
            Session.library_modified = 1
        return 1
