# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from NumberQuestion import NumberQuestion


class Choice(NumberQuestion):
    """A simple choice."""

    __title = ""

    __items = []
    
    def __init__(self, p_title, p_prompt, p_default = 1):
        """Construct a choice."""
        NumberQuestion.__init__(self, p_prompt, str(p_default))
        self.__title = p_title
        self.__items = []

    def add_item(self, p_item):
        """Add an item."""
        self.__items.append(p_item)

    def get_item(self, p_num):
        """Return the item of the given index."""
        _ret = None
        if p_num >=0 and p_num < len(self.__items):
            _ret = self.__items[p_num]
        return _ret

    def remove_items(self):
        """Remove all items."""
        self.__items = []

    def get_item_as_string(self, _item):
        """Return the given item as a string."""
        return str(_item)

    def get_items_number(self):
        """Return the number of items."""
        return len(self.__items)

    def has_no_item(self):
        """Determine if there is no item."""
        return (self.get_items_number() <= 0)

    def show_items(self):
        """Show all items."""
        _len = self.get_items_number()
        if _len < 1:
            raise ChoiceException, 'Not enough items: %d' % _len
        print self.__title, ":"
        _max = len(self.__items)
        _max_num = len(str(_max))
        _cpt = 1
        for _item in self.__items:
            _text = "[%*d] " % (_max_num, _cpt)
            _text += self.get_item_as_string(_item)
            print _text
            _cpt += 1
        
    def show(self):
        """Print the choice and return the response as an integer."""
        print
        self.show_items()
        _choice = 0
        if len(self.__items) > 0:
            # asks the question until the song number is correct
            while _choice < 1 or _choice > len(self.__items):
                _choice = NumberQuestion.show(self)
        return _choice - 1


class ChoiceException(Exception):
    """Exception raised when there are less than two items for the choice."""

    pass

