# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from base.Manager import Manager
from base.PathConfig import get_simple_example
from base.PathConfig import get_complex_example
from base.PathConfig import PathConfig

from Menu import Menu
from Question import Question
from SongFileDisplay import SongFileDisplay


class ConfigurationMenu(Menu):
    "Menu which allows to modify user preferences."
    
    def __init__(self):
        "Constructs the menu."
        Menu.__init__(self, "Configuration", "Your choice")
        self.add_entry("Change the temporary directory", self.__change_tmp)
        self.add_entry("Change the file path naming", self.__change_naming)
        self.add_entry("Go back", self.go_back)

    def __change_tmp(self):
        _manager = Manager()
        _tmp_path = _manager.get_tmp_path()
        _quest = Question('New temporary directory path', _tmp_path)
        _resp = _quest.show()
        _manager.set_tmp_path(_resp)
        return 1

    def __change_naming(self):
        _manager = Manager()
        _path_config = PathConfig()
        # change main format
        _format = _manager.get_path_config().get_format()
        _quest = Question('Main format', _format)
        _resp = _quest.show()
        _path_config.set_format(_resp)
        # change disc format
        _format = _manager.get_path_config().get_disc_format()
        _quest = Question('Disc format', _format)
        _resp = _quest.show()
        _path_config.set_disc_format(_resp)
        # change artists separator
        _sep = _manager.get_path_config().get_artists_sep()
        _quest = Question('Artists separator', _sep)
        _resp = _quest.show()
        _path_config.set_artists_sep(_resp)
        # change space substitute
        _sub = _manager.get_path_config().get_space_substitute()
        _quest = Question('Space substitute', _sub)
        _resp = _quest.show()
        _path_config.set_space_substitute(_resp)
        # change lowercasing
        _lower = _manager.get_path_config().is_lowercasing()
        if _lower:
            _default = 'y'
        else:
            _default = 'n'
        _quest = Question('Enable lowercasing', _default)
        while _resp != 'y' and _resp != 'n':
            _resp = _quest.show()
        if _resp == 'y':
            _lower = 1
        else:
            _lower = 0
        _path_config.set_lowercasing(_lower)
        # example
        print 'Here are two examples of the new file path naming'
        print
        print 'Simple example'
        _file = get_simple_example(_path_config)
        _display = SongFileDisplay(_file)
        _path = _file.get_path()
        print 'Song :'
        print _display
        print 'Path :'
        print _path
        print
        print 'Complex example'
        _file = get_complex_example(_path_config)
        _display = SongFileDisplay(_file)
        _path = _file.get_path()
        print 'Song :'
        print _display
        print 'Path :'
        print _path
        _manager.set_path_config(_path_config)
        return 1

