# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from base.Manager import Manager
from base.Artist import Artist
from base.Disc import Disc
from base.Tune import Tune
from base.Song import get_empty_song
from base.Song import Song

from Question import Question
from NumberQuestion import NumberQuestion
from Choice import Choice


class Edition:
    "Set of questions to edit a song."

    __song = None

    def __init__(self, p_song = None):
        if p_song is None:
            self.__song = get_empty_song()
        else:
            self.__song = p_song

    def ask_for_singers(self):
        "Displays some questions to edit the main singers."
        _new_artist = None
        _old_artist = self.__song.get_artist()
        _len = 0
        if _old_artist is not None:
            _len = _old_artist.get_names_number()
        _cpt = 0
        _end = 0
        while not _end:
            if _cpt < _len:
                _name = _old_artist.get_name(_cpt)
            else:
                _name = ''
            # display an artist singer entry
            _str = 'Song singer ' + str(_cpt + 1)
            _quest = Question(_str, _name)
            _name = _quest.show()
            if _name != '':
                if _new_artist is None:
                    # create an artist if it the first name
                    _new_artist = Artist(_name)
                else:
                    # add the name to the artist
                    _new_artist.add_name(_name)
            else:
                # stop the singers edition
                _end = 1
            _cpt += 1
        # update the song singers
        self.__song.set_artist(_new_artist)

    #########
    # TO DO #
    #########
    def ask_for_feat_singers(self):
        "Displays some questions to edit the featuring singers."
        pass

    def ask_for_title(self):
        "Displays a question to edit the song title."
        _tune = self.__song.get_tune()
        _quest = Question("Song title", _tune.get_title())
        _title = _quest.show()
        _tune.set_title(_title)
        self.__song.set_tune(_tune)

    def ask_for_disc(self):
        "Displays some questions to edit the disc."
        _ret = None
        _manager = Manager()
        _disc = self.__song.get_disc()
#        _library = _manager.get_library_selection()
        _discs = _manager.get_library_selection().search_disc(_disc)
        _resp = len(_discs)
        if len(_discs) > 0:
            _choice = Choice("Availlable discs", "Your choice", len(_discs) + 1)
            for _disc in _discs:
                _choice.add_item(str(_disc))
            _choice.add_item("Other...")
            _resp = _choice.show()
        if _resp != len(_discs):
            _ret = _discs[_resp]
            self.__song.set_disc(_ret)
        return _ret
      
    def ask_for_disc_title(self):
        _disc = self.__song.get_disc()
        _quest = Question("Disc title", _disc.get_title())
        _title = _quest.show()
        _disc = Disc(_title)
        self.__song.set_disc(_disc)
        return _title

    def ask_for_an_album(self):
        _ret = 0
        if self.__song.get_disc().has_artist():
            _default = "y"
        else:            
            _default = "n"
        _quest = Question("Is it an album ?", _default)
        _res = _quest.show()
        if _res == "y":
            _ret = 1
        return _ret

    def set_disc_as_album(self):
        _disc = self.__song.get_disc()
        _artist = self.__song.get_artist().get_main_artist()
        _disc.set_artist(_artist)
        """
        _singers = self.__song.get_singers()
        if len(_singers) > 0:
            _disc.set_artist(_singers[0])
        """

    def set_disc_not_as_album(self):
        self.__song.get_disc().set_no_artist()

    def ask_for_track(self):
        "Displays a question to edit the song track."
        _quest = Question("Track", self.__song.get_track())
        _track = _quest.show()
        self.__song.set_track(_track)

    def ask_for_year(self):
        "Displays a question to edit the song year."
        _quest = Question("Year", self.__song.get_tune().get_year())
        _year = _quest.show()
        self.__song.get_tune().set_year(_year)

    def ask_for_genre(self):
        "Displays a question to edit the song genre."
        _quest = Question("Genre", self.__song.get_genre())
        _genre = _quest.show()
        self.__song.set_genre(_genre)

    def ask_for_comment(self):
        "Displays a question to edit the song comment."
        _quest = Question("Comment", self.__song.get_comment())
        _comment = _quest.show()
        self.__song.set_comment(_comment)

    def show(self):
        "Displays some questions to edit the song."
        self.ask_for_singers()
        self.ask_for_title()
        if not self.ask_for_disc():
            self.ask_for_disc_title()
            if self.ask_for_an_album():
                self.set_disc_as_album()
            else:
                self.set_disc_not_as_album()
        self.ask_for_track()
        self.ask_for_year()
        self.ask_for_genre()
        self.ask_for_comment()

    def get_song(self):
        return self.__song
