# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from util.Directory import Directory

from base.Manager import Manager
from base.SongEditing import get_song_editing_tools
from base.SongEditing import can_edit_song
from base.LibraryRefreshing import get_library_refreshing_tools
from base.LibraryRefreshing import can_refresh_library

from Menu import Menu
from AddMenu import AddMenu
from Question import Question
from Edition import Edition
from SelectionList import SelectionList
from MissingToolsMessage import MissingToolsMessage
from Session import Session


class LibraryMenu(Menu):
    "Menu which allows to manage the library."
    
    def __init__(self):
        "Constructs the menu."
        Menu.__init__(self, "Management of the library", "Your choice")
        self.add_entry("List all songs", self.__list)
        self.add_entry("Add songs", self.__add_song)
        self.add_entry("Edit a song", self.__edit_song)
        self.add_entry("Remove a song", self.__remove_song)
        self.add_entry("Refresh the library", self.__refresh_library)
        self.add_entry("Put a song in the play queue", self.__put_song)
        self.add_entry("Go back", self.go_back)

    def __list(self):
        _manager = Manager()
        _sel = _manager.get_library_selection()
        if _sel.is_empty():
            print 'No song in library'
        else:
            _list = SelectionList(_sel)
            _list.show_items()
        return 1


    def __add_song(self):
        _menu = AddMenu()
        _menu.show()
        return 1

    def __edit_song(self):
        if not can_edit_song():
            _error = 'Sorry but you edit a song.'
            _tools = get_song_editing_tools()
            _msg = MissingToolsMessage(_error, _tools)
            _msg.display()
        else:
            _manager = Manager()
            _sel = _manager.get_library_selection()
            if _sel.is_empty():
                print 'No song in library'
            else:
                # asks for a song
                _list = SelectionList(_sel)
                _file = _list.ask_for_file()
                if _file:
                    # asks for editing a song
                    # get a copy of the song
                    _song = _file.get_song()
                    _song = _song.clone()
                    # edit the song
                    _edit = Edition(_song)
                    _edit.show()
                    _song = _edit.get_song()
                    # updates the song
                    _task = _manager.get_song_editing(_file.get_path(), _song)
                    _manager.get_tasks_thread().add_task(_task)
                    print "The file", _file.get_path(), "will be updated"
                    Session.library_modified = 1
                    Session.play_lists_modified = 1
        return 1

    def __remove_song(self):
        _manager = Manager()
        _sel = _manager.get_library_selection()
        if _sel.is_empty():
            print 'No song in library'
        else:
            # asks for a song
            _list = SelectionList(_sel)
            _file = _list.ask_for_file()
            if _file:
                # asks for a confirmation
                _quest = Question("Remove the file " + _file.get_path() + " ?", "n")
                _resp = _quest.show()
                if _resp == "y":
                    # removes the song
                    _task = _manager.get_song_removing(_file.get_path())
                    _manager.get_tasks_thread().add_task(_task)
                    print "The file %s will be removed" % _file.get_path()
                    Session.library_modified = 1
                    Session.play_lists_modified = 1
        return 1

    def __refresh_library(self):
        if not can_refresh_library():
            _error = 'Sorry but you cannot refresh library.'
            _tools = get_library_refreshing_tools()
            _msg = MissingToolsMessage(_error, _tools)
            _msg.display()
        else:
            _manager = Manager()
            _path = _manager.get_root_path()
            _quest = Question("Root directory", _path)
            _resp = _quest.show()
            _dir = Directory(_resp)
            if _dir.exists():
                _task = _manager.get_library_refreshing(_resp)
                _manager.get_tasks_thread().add_task(_task)
                print "The library will be refreshed"
                Session.library_modified = 1
            else:
                print "The directory doesn't exist."
        return 1

    def __put_song(self):
        _manager = Manager()
        _sel = _manager.get_library_selection()
        if _sel.is_empty():
            print 'No song in library'
        else:
            # asks for a song
            _list = SelectionList(_sel)
            _file = _list.ask_for_file()
            if _file:
                _manager.get_play_thread().get_queue().append(_file)
                print "The song has been added to the play queue"
        return 1

