# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from Choice import Choice


class Menu(Choice):
    "A simple menu of commands."
    
    def __init__(self, p_title, p_prompt, p_default = "1"):
        "Construct a menu."
        Choice.__init__(self, p_title, p_prompt, p_default)

    
    def add_entry(self, p_label, p_callback):
        "Add a menu entry."
        _entry = MenuEntry(p_label, p_callback)
        Choice.add_item(self, _entry)

    def remove_items(self):
        "Remove all items."
        Choice.remove_items(self)

    def get_item_as_string(self, _item):
        "Return the given item as a string."
        return _item.get_label()

    def show(self):
        "Print the menu."
        _res = 1
        while _res:
            _choice = Choice.show(self)
            _method = self.get_item(_choice).get_callback()
            _res = _method()
        return _res

    def go_back(self):
        return 0

class MenuEntry:
    "An entry in the menu."

    __label = ""

    __callback = None

    def __init__(self, p_label, p_callback):
        "Constructs an entry."
        self.__label = p_label
        self.__callback = p_callback

    def get_label(self):
        "Gets the label."
        return self.__label

    def get_callback(self):
        "Gets the callback."
        return self.__callback

