# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from base.Manager import Manager

from Menu import Menu
from SelectionMenu import SelectionMenu
from SelectionList import SelectionList
from Session import Session


class PlayListMenu(Menu):
    "Menu which allows to manage a play list."

    __play_list = None
    
    def __init__(self, p_play_list):
        "Constructs the menu."
        Menu.__init__(self, "Management of a play list", "Your choice")
        self.add_entry("List songs", self.__list)
        self.add_entry("Add a song", self.__add_song)
        self.add_entry("Remove a song", self.__remove_song)
        self.add_entry("Search in the library", self.__search)
        self.add_entry("Sort", self.__sort)
        self.add_entry("Put all songs in the play queue", self.__play_songs)
        self.add_entry("Go back", self.go_back)
        self.__play_list = p_play_list

    def __list(self):
        _len = self.__play_list.get_length()
        if _len < 1:
            print
            print "The play list is empty."
        else:
            _list = SelectionList(self.__play_list)
            _list.show_items()
        return 1

    def __add_song(self):
        _manager = Manager()
        _sel = _manager.get_library_selection()
        if _sel.is_empty():
            print 'No song in library'
        else:
            # lists all songs availlable
            _list = SelectionList(_manager.get_library_selection())
            _file = _list.ask_for_file()
            # adds the file name to the play list
            self.__play_list.append(_file)
            Session.play_lists_modified = 1
        return 1

    def __remove_song(self):
        if self.__play_list.is_empty():
            print "The play list is empty."
        else:
            _list = SelectionList(self.__play_list)
            _num = _list.ask_for_number()
            self.__play_list.delete(_num)
            Session.play_lists_modified = 1
        return 1

    def __search(self):
        _menu = SelectionMenu()
        _menu.show()
        _sel = _menu.get_selection()
        if _sel is not None:
            self.__play_list.extend(_sel)
            Session.play_lists_modified = 1
        return 1

    def __sort(self):
        _menu = Menu("Criterion", "Your choice")
        _menu.add_entry("Title", self.__play_list.sort_by_title)
        _menu.add_entry("Artist", self.__play_list.sort_by_artist)
        _menu.add_entry("Year", self.__play_list.sort_by_year)
        _menu.add_entry("Disc", self.__play_list.sort_by_disc)
        _menu.add_entry("Track", self.__play_list.sort_by_track)
        _menu.add_entry("Genre", self.__play_list.sort_by_genre)
        _menu.show()
        Session.play_lists_modified = 1
        return 1

    def __play_songs(self):
        _manager = Manager()
        _queue = _manager.get_play_thread().get_queue()
        _queue.extend(self.__play_list)
        print "The play list has been added in the queue."
        return 1
