# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from base.Manager import Manager
from base.PlayList import PlayList

from Menu import Menu
from Question import Question
from Choice import Choice
from PlayListMenu import PlayListMenu
from Session import Session


class PlayListsSetMenu(Menu):
    "Menu to manage all play lists."

    def __init__(self):
        "Constructs the menu."
        Menu.__init__(self, "Management of all play lists", "Your choice")
        self.add_entry("Create a new play list", self.__create)
        self.add_entry("Manage a play list", self.__manage)
        self.add_entry("Remove a play list", self.__remove)
        self.add_entry("Go back", self.go_back)

    def __create(self):
        _title = ""
        # asks for a title
        _quest = Question("Title of the new play list")
        while _title == "":
            _title = _quest.show()
        # creates a new playlist
        _play_list = PlayList(_title)
        _manager = Manager()
        _manager.get_play_lists_set().append(_play_list)
        # displays the menu
        _menu = PlayListMenu(_play_list)
        _menu.show()
        Session.play_lists_modified = 1
        return 1

    def __manage(self):
        _sel = self.__choose_play_list()
        if _sel is not None:
            # displays the menu to manage the play list
            _menu = PlayListMenu(_sel)
            _menu.show()
        return 1

    def __remove(self):
        _sel = self.__choose_play_list()
        if _sel is not None:
            _title = _sel.get_title()
            _quest = Question("Are you sure to remove the play list " + _title, "n")
            if _quest.show() == "y":
                _manager = Manager()
                _manager.get_play_lists_set().remove_play_list(_title)
                print "The play list %s has been removed" % _title
                Session.play_lists_modified = 1
        return 1

    def __choose_play_list(self):
        _ret = None
        _manager = Manager()
        _play_lists_set = _manager.get_play_lists_set()
        if _play_lists_set.is_empty():
            print "No play list"
        else:
            _choice = Choice("Current play lists", "Your choice", 1)
            for _pl in _play_lists_set:
                _choice.add_item(_pl.get_title())
            _num = _choice.show()
            _ret = _play_lists_set[_num]
        return _ret
    
