# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

from base.Manager import Manager
from base.Process import ToolNotFoundException

from Menu import Menu
from SelectionList import SelectionList
from SongFileDisplay import SongFileDisplay


class PlayQueueMenu(Menu):
    "menu to manange the songs playing."

    def __init__(self):
        "Constructs the main menu."
        Menu.__init__(self, "Management of the play queue", "Your choice")
        self.__make()

    def __make(self):
        self.remove_items()
        self.add_entry('List the queue', self.__list)
        self.add_entry('View current song', self.__view)
        self.add_entry('Stop', self.__stop)
        self.add_entry('Play next', self.__play_next)
#        self.add_entry("Play next song", self.__play_next)
        _manager = Manager()
        _thread = _manager.get_play_thread()
        if _thread.get_looping():
            self.add_entry('Disable looping', self.__disable_looping)
        else:
            self.add_entry('Enable looping', self.__enable_looping)
        self.add_entry('Randomize', self.__randomize)
        self.add_entry('Go back', self.go_back)

    def __list(self):
        _manager = Manager()
        _queue = _manager.get_play_thread().get_queue()
        if _queue.is_empty():
            print 'The queue is empty.'
        else:
            _list = SelectionList(_queue)
            _list.show_items()
        return 1

    def __view(self):
        _manager = Manager()
        _file = _manager.get_play_thread().get_current_file()
        if _file:
            _display = SongFileDisplay(_file)
            print 'Current song : %s' % str(_display)
        else:
            print 'No current song'
        return 1

    def __stop(self):
        _manager = Manager()
        _manager.get_play_thread().stop()
        return 1

    def __play_next(self):
        _manager = Manager()
        try:
            _manager.get_play_thread().play_next()
        except ToolNotFoundException, _exc:
            print 'Error : the tool %s is not available on your system.' % _exc
        return 1

    def __set_looping(self, p_bool):
        _manager = Manager()
        _manager.get_play_thread().set_looping(p_bool)
        self.__make()

    def __disable_looping(self):
        self.__set_looping(0)
        return 1

    def __enable_looping(self):
        self.__set_looping(1)
        return 1

    def __randomize(self):
        _manager = Manager()
        _manager.get_play_thread().get_queue().random()
        return 1
