# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


class SongFileDisplay:
    """A song file display."""

    __song_file = None

    __sep = ' - '

    def __init__(self, p_song_file, p_sep = ' - '):
        self.__song_file = p_song_file
        self.__sep = p_sep

    def __format(self, p_object, p_width, p_right_align = 0):
        """Return a representation of the given object.
        If the given string is lesser than the given width,
        white spaces are added.
        If the given string is greater than the given width,
        the string is truncated with thre points at its end.
        """
        _str = str(p_object)
        _len = len(_str)
        if _len < p_width:
            _spaces = ' ' * (p_width - _len) 
            if p_right_align:
                # add spaces at the beginning
                _str = _spaces + _str
            else:
                # add spaces at the end
                _str += _spaces
        if _len > p_width:
            # trucate and add three points
            _str = _str[:p_width - 3] + '...'
        return _str

    def __str__(self):
        """Return the representation of a song file as a string."""
        _ret = ''
        _song = self.__song_file.get_song()
        if _song is None:
            _ret += self.__song_file.get_path()
        else:
            """
            _ret += '%-30s %-20s %-25s %-5s %-5s %-15s' % (_song.get_tune().get_title(),
                                                           _song.get_artist(),
                                                           _song.get_disc(),
                                                           _song.get_tune().get_year(),
                                                           _song.get_track(),
                                                           _song.get_genre())
            """
            _ret += self.__format(_song.get_tune().get_title(), 15, 0) + self.__sep
            _ret += self.__format(_song.get_artist()          , 10, 0) + self.__sep
            _ret += self.__format(_song.get_disc()            , 15, 0) + self.__sep
            _ret += self.__format(_song.get_tune().get_year() , 4 , 1) + self.__sep
            _ret += self.__format(_song.get_track()           , 2 , 1) + self.__sep
            _ret += self.__format(_song.get_genre()           , 13, 0)
        return _ret

