# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


from util.UserException import UserException
from util.File import FileNotFoundException
from util.File import FileAlreadyExistsException
from util.File import FileNotReadableException
from util.File import FileNotWritableException
from util.Directory import DirectoryNotFoundException
from util.Directory import DirectoryAlreadyExistsException
from util.Directory import DirectoryNotReadableException
from util.Directory import DirectoryNotEmptyException
from util.Directory import DirectoryNotWritableException
from util.Directory import DirectoryNotAccessibleException

from base.Process import BadStatusException
from base.MusicFile import FileNotOggVorbisException
from base.MusicFile import FileNotMP3Exception
from base.LibraryLoading import LibraryParserException
from base.Manager import Manager

from Menu import Menu
from Question import Question
from TasksList import TasksList


class TasksMenu(Menu):
    "Menu which allows to modify the tasks queue."
    
    def __init__(self):
        "Constructs the menu."
        Menu.__init__(self, "Management of the tasks queue", "Your choice")
        self.add_entry("List all tasks", self.__list)
        self.add_entry("Cancel a task", self.__cancel)
        self.add_entry("View errors", self.__view_errors)
        self.add_entry("Go back", self.go_back)

    def __list(self):
        _list = TasksList()
        if _list.has_no_item():
            print 'No task'
        else:
            _list.show_items()
        return 1

    def __cancel(self):
        # asks for a file name
        _list = TasksList()
        if _list.has_no_item():
            print 'No task to cancel'
        else:
            _number = _list.show()
            # asks for a confirmation
            _quest = Question('Cancel the task %d ?' % (_number + 1), 'n')
            _resp = _quest.show()
            if _resp == 'y':
                # cancels the task
                _manager = Manager()
                _manager.get_tasks_thread().cancel_task(_number)
                print "The task %d has been cancelled" % _number
        return 1

    def __get_error_message(self, p_error):
        _ret = ''
        try:
            raise p_error
        except FileNotFoundException, _exc:
            _ret = 'The file %s does not exist' % _exc
        except FileNotReadableException, _exc:
            _ret = 'The file %s cannot be read' % _exc
        except FileNotWritableException, _exc:
            _ret = 'The file %s cannot be written' % _exc
        except DirectoryNotFoundException, _exc:
            _ret = 'The directory %s does not exist' % _exc
        except DirectoryAlreadyExistsException, _exc:
            _ret = 'The directory %s does already exist' % _exc
        except DirectoryNotReadableException, _exc:
            _ret = 'The directory %s cannot be read' % _exc
        except DirectoryNotEmptyException, _exc:
            _ret = 'The directory %s is not empty' % _exc
        except DirectoryNotWritableException, _exc:
            _ret = 'The directory %s cannot be written' % _exc
        except DirectoryNotAccessibleException, _exc:
            _ret = 'The directory %s cannot be accessed' % _exc
        except BadStatusException, _exc:
            _ret = 'A process results a bad status %s' % _exc
        except FileNotOggVorbisException, _exc:
            _ret = 'The file %s is not an Ogg Vorbis file' % _exc
        except FileNotMP3Exception, _exc:
            _ret = 'The file %s is not a MP3 file' % _exc
        except LibraryParserException, _exc:
            _ret = 'A XML parse error occured in file %s at line %s and column %s : %s - %s' % (_exc.get_file_path(), _exc.get_line(), _exc.get_column(), _exc.get_code(), _exc.get_message())
        except UserException, _exc:
            _ret = 'An unknown error occured : %s' % _exc
        return _ret


    def __view_errors(self):
        _manager = Manager()
        _errors = _manager.get_tasks_thread().get_errors()
        if len(_errors) == 0:
            print "No error"
        else:
            for _err in _errors:
                print self.__get_error_message(_err)
        return 1

    
