# maestro - An Ogg Vorbis files manager
#
# Copyright (c) 2002, Julien Lesouef <julien.lesouef@free.fr>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import os
import xml.sax.saxutils

INDENT_SPACES_NUMBER = 3


class XMLOutput:
    "A XML document output."""

    __generator = None

    __indent = 0

    def __init__(self, p_file):
        self.__generator = xml.sax.saxutils.XMLGenerator(p_file)
        self.__indent = 0

    def get_indent(self):
        """Return the number of indentation space."""
        return self.__indent

#    def __convert_data(self, p_data):
#        return xml.sax.saxutils.escape(p_data)

    def __write_eol(self):
        """Write an end of line."""
        self.__generator.characters(os.linesep)
 
    def __write_indent(self):
        """Write  spaces to indent."""
        self.__generator.characters(' ' * self.__indent)

    def write_begin_root_tag(self, p_tag, p_att = {}):
        """Write the beginning of the document, the root start element and
        its attributes in the given dictionnry."""
        self.__generator.startDocument()
        self.write_begin_tag(p_tag, p_att)
        
    def write_begin_tag(self, p_tag, p_att = {}):
        """Write a start element and its attributes in the given dictionnry."""
        self.__write_indent()
        self.__generator.startElement(p_tag, p_att)
        self.__write_eol()
        self.__indent += INDENT_SPACES_NUMBER
        
    def write_tags_with_data(self, p_tag, p_data, p_att = {}):
        """Write a start element, data and an end element."""
        self.__write_indent()
        self.__generator.startElement(p_tag, p_att)
        self.__generator.characters(p_data)
        self.__generator.endElement(p_tag)
        self.__write_eol()

    def write_end_tag(self, p_tag):
        """Write an end element."""
        self.__indent -= INDENT_SPACES_NUMBER
        self.__write_indent()
        self.__generator.endElement(p_tag)
        self.__write_eol()
    
    def write_end_root_tag(self, p_tag):
        """Write the the end start element."""
        self.write_end_tag(p_tag)
        self.__generator.endDocument()

if __name__ == '__main__':
    import sys
    _writer = XMLOutput(sys.stdout)
    _writer.write_begin_root_tag('root')
    _writer.write_begin_tag('chapter')
    _writer.write_tags_with_data('lesson', 'First section & first lesson', {'no':'1'})
    _writer.write_tags_with_data('lesson', 'The statement "2 < 1" is false', {'no':'2'})
    _writer.write_end_tag('chapter')
    _writer.write_tags_with_data('chapter', 'Second section')
    _writer.write_end_root_tag('root')

