#!/usr/bin/env python

import sys
import os
import tempfile
from distutils.core import setup
from distutils import sysconfig

from maestro.base.__init__ import __project_name__
from maestro.base.__init__ import __version__

LONG_DESCRIPTION = \
"""Maestro is an application for managing an Ogg Vorbis files library.
It can be considered as a front-end for many command-line tools like
ogg123, oggenc, cdparanoia, etc..."""

SCRIPTS = ['maestro-text', 'maestro-gui']

def remove_file(p_file_path):
    """Try to remove the given file path, even if it does not exist."""
    try :
        os.unlink(p_file_path)
    except :
        pass

if __name__ == '__main__':

    print 'creating utility scripts...'
    _install_path = '/usr/lib' + os.sep + 'maestro'
    _python_exec_path = sys.executable
    _tmp_scripts = []

    for _tool in SCRIPTS:
        _tool_path = _install_path + os.sep + _tool + '.py'

        # construct the script content
        _script_str = '#!/bin/sh\n\n%s %s "$@"\n' % (_python_exec_path, _tool_path)
        if sys.platform == 'win32' :
            _script_str = '%s %s %%*\n' % (_python_exec_path, _tool_path)

        # determine the script suffix
        _script_suffix = ''
        if sys.platform == 'win32' :
            _script_suffix = '.bat'

        # construct the script path 
        _script_name = _tool + _script_suffix
        _script_path = os.path.join(tempfile.gettempdir(), _script_name)

        # remove the script path if already exist
        remove_file(_script_path)

        # write the script content at the path
        try :
            fp = open(_script_path, "w")
            fp.write(_script_str)
            fp.close()
            if sys.platform != 'mac' :
                os.chmod(_script_path, 0755)
        except:
            print 'Unable to create utility script : %s' % _tool
            raise

        _tmp_scripts.append(_script_path)
    
    print 'scripts created : %s' % _tmp_scripts

    print 'creating text documentation...'
    os.system('python doc/maestro-doc.py TXT . doc/project.xml')
#    print 'creating html documentation...'
#    os.system('python doc/maestro-doc.py HTML doc/html doc/project.xml')

    setup (name = __project_name__,
           version = __version__,
           licence = 'GNU GPL',
           description = 'Ogg Vorbis files manager',
           long_description = LONG_DESCRIPTION,
           
           author = 'Julien Lesouef',
           author_email = 'julien.lesouef@free.fr',
           url = 'http://upsilon.free.fr/maestro/',

           packages = ['maestro',
                       'maestro.util',
                       'maestro.base',
                       'maestro.text',
                       'maestro.gui'],
           scripts = _tmp_scripts,
#           data_files = [('/usr/share/doc', ('AUTHORS', 'BUGS', 'CHANGES', 'DEVEL', 'INSTALL', 'LICENSE', 'NEWS', 'README', 'TODO', ))],
           )

    print 'removing utility scripts...'
    # cleanup old script file, it should have been installed
    for _path in _tmp_scripts:
        remove_file(_path)
